@@ -1,22 +1,10 @@
 /*
- *  Copyright (c) 2014 Dmitry Kazakov <dimula73@gmail.com>
+ *  SPDX-FileCopyrightText: 2014 Dmitry Kazakov <dimula73@gmail.com>
  *
  *  See LayerFX plugin for Gimp as a reference implementation of this style:
  *  http://registry.gimp.org/node/186
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "kis_ls_bevel_emboss_filter.h"
@@ -50,6 +38,7 @@
 #include "gimp_bump_map.h"
 #include "kis_transaction.h"
 #include "kis_multiple_projection.h"
+#include "kis_cached_paint_device.h"
 
 
 KisLsBevelEmbossFilter::KisLsBevelEmbossFilter()
@@ -57,19 +46,32 @@ KisLsBevelEmbossFilter::KisLsBevelEmbossFilter()
 {
 }
 
+KisLayerStyleFilter *KisLsBevelEmbossFilter::clone() const
+{
+    return new KisLsBevelEmbossFilter(*this);
+}
+
+KisLsBevelEmbossFilter::KisLsBevelEmbossFilter(const KisLsBevelEmbossFilter &rhs)
+    : KisLayerStyleFilter(rhs)
+{
+}
+
 void paintBevelSelection(KisPixelSelectionSP srcSelection,
                          KisPixelSelectionSP dstSelection,
                          const QRect &applyRect,
                          int size,
                          int initialSize,
-                         bool invert)
+                         bool invert,
+                         KisLayerStyleFilterEnvironment *env)
 {
-    KisSelectionSP tmpBaseSelection = new KisSelection(new KisSelectionEmptyBounds(0));
+    KisCachedSelection::Guard s1(*env->cachedSelection());
+    KisSelectionSP tmpBaseSelection = s1.selection();
     KisPixelSelectionSP tmpSelection = tmpBaseSelection->pixelSelection();
 
     // NOTE: we are not using createCompositionSourceDevice() intentionally,
     //       because the source device doesn't have alpha channel
-    KisPixelSelectionSP fillDevice = new KisPixelSelection();
+    KisCachedSelection::Guard s2(*env->cachedSelection());
+    KisPixelSelectionSP fillDevice = s2.selection()->pixelSelection();
 
     KisPainter gc(dstSelection);
     gc.setCompositeOp(COMPOSITE_COPY);
@@ -148,11 +150,11 @@ void mapPixelValues(KisPixelSelectionSP srcSelection,
     KisSequentialConstIterator srcIt(srcSelection, applyRect);
     KisSequentialIterator dstIt(dstSelection, applyRect);
 
-    do {
+    while (srcIt.nextPixel() && dstIt.nextPixel()) {
         const quint8 *srcPtr = srcIt.rawDataConst();
         quint8 *dstPtr = dstIt.rawData();
         *dstPtr = mapTable[*srcPtr];
-    } while(srcIt.nextPixel() && dstIt.nextPixel());
+    }
 }
 
 template <class MapOp>
@@ -173,10 +175,10 @@ void mapPixelValues(KisPixelSelectionSP dstSelection,
 
     KisSequentialIterator dstIt(dstSelection, applyRect);
 
-    do {
+    while (dstIt.nextPixel()) {
         quint8 *dstPtr = dstIt.rawData();
         *dstPtr = mapTable[*dstPtr];
-    } while(dstIt.nextPixel());
+    }
 }
 
 struct BevelEmbossRectCalculator
@@ -260,34 +262,37 @@ void KisLsBevelEmbossFilter::applyBevelEmboss(KisPaintDeviceSP srcDevice,
                                               KisMultipleProjection *dst,
                                               const QRect &applyRect,
                                               const psd_layer_effects_bevel_emboss *config,
+                                              KisResourcesInterfaceSP resourcesInterface,
                                               KisLayerStyleFilterEnvironment *env) const
 {
     if (applyRect.isEmpty()) return;
 
     BevelEmbossRectCalculator d(applyRect, config);
 
-    KisSelectionSP baseSelection = KisLsUtils::selectionFromAlphaChannel(srcDevice, d.initialFetchRect);
-    KisPixelSelectionSP selection = baseSelection->pixelSelection();
+    KisCachedSelection::Guard s1(*env->cachedSelection());
+    KisSelectionSP baseSelection = s1.selection();
+    KisLsUtils::selectionFromAlphaChannel(srcDevice, baseSelection, d.initialFetchRect);
 
-    //selection->convertToQImage(0, QRect(0,0,300,300)).save("0_selection_initial.png");
+    KisPixelSelectionSP selection = baseSelection->pixelSelection();
 
     const int size = config->size();
 
     int limitingGrowSize = 0;
-    KisPixelSelectionSP bumpmapSelection = new KisPixelSelection(new KisSelectionEmptyBounds(0));
+    KisCachedSelection::Guard s2(*env->cachedSelection());
+    KisPixelSelectionSP bumpmapSelection = s2.selection()->pixelSelection();
 
     switch (config->style()) {
     case psd_bevel_outer_bevel:
-        paintBevelSelection(selection, bumpmapSelection, d.applyBevelRect, size, size, false);
+        paintBevelSelection(selection, bumpmapSelection, d.applyBevelRect, size, size, false, env);
         limitingGrowSize = size;
         break;
     case psd_bevel_inner_bevel:
-        paintBevelSelection(selection, bumpmapSelection, d.applyBevelRect, size, 0, false);
+        paintBevelSelection(selection, bumpmapSelection, d.applyBevelRect, size, 0, false, env);
         limitingGrowSize = 0;
         break;
     case psd_bevel_emboss: {
         const int initialSize = std::ceil(qreal(size) / 2.0);
-        paintBevelSelection(selection, bumpmapSelection, d.applyBevelRect, size, initialSize, false);
+        paintBevelSelection(selection, bumpmapSelection, d.applyBevelRect, size, initialSize, false, env);
         limitingGrowSize = initialSize;
         break;
     }
@@ -295,8 +300,8 @@ void KisLsBevelEmbossFilter::applyBevelEmboss(KisPaintDeviceSP srcDevice,
         const int halfSizeF = std::floor(qreal(size) / 2.0);
         const int halfSizeC = std::ceil(qreal(size) / 2.0);
         // TODO: probably not correct!
-        paintBevelSelection(selection, bumpmapSelection, d.applyBevelRect, halfSizeC, halfSizeC, false);
-        paintBevelSelection(selection, bumpmapSelection, d.applyBevelRect, halfSizeF, 0, true);
+        paintBevelSelection(selection, bumpmapSelection, d.applyBevelRect, halfSizeC, halfSizeC, false, env);
+        paintBevelSelection(selection, bumpmapSelection, d.applyBevelRect, halfSizeF, 0, true, env);
         limitingGrowSize = halfSizeC;
         break;
     }
@@ -305,7 +310,9 @@ void KisLsBevelEmbossFilter::applyBevelEmboss(KisPaintDeviceSP srcDevice,
         return;
     }
 
-    KisPixelSelectionSP limitingSelection = new KisPixelSelection(*selection);
+    KisCachedSelection::Guard s3(*env->cachedSelection());
+    KisPixelSelectionSP limitingSelection = s3.selection()->pixelSelection();
+    limitingSelection->makeCloneFromRough(selection, selection->selectedRect());
     {
         QRect changeRectUnused =
             KisLsUtils::growSelectionUniform(limitingSelection,
@@ -317,11 +324,12 @@ void KisLsBevelEmbossFilter::applyBevelEmboss(KisPaintDeviceSP srcDevice,
     //bumpmapSelection->convertToQImage(0, QRect(0,0,300,300)).save("1_selection_xconv.png");
 
     if (config->textureEnabled()) {
-        KisPixelSelectionSP textureSelection = new KisPixelSelection(new KisSelectionEmptyBounds(0));
+        KisCachedSelection::Guard s4(*env->cachedSelection());
+        KisPixelSelectionSP textureSelection = s4.selection()->pixelSelection();
 
         KisLsUtils::fillPattern(textureSelection, d.applyTextureRect, env,
                                 config->textureScale(),
-                                config->texturePattern(),
+                                config->texturePattern(resourcesInterface),
                                 config->textureHorizontalPhase(),
                                 config->textureVerticalPhase(),
                                 config->textureAlignWithLayer());
@@ -399,7 +407,7 @@ void KisLsBevelEmbossFilter::applyBevelEmboss(KisPaintDeviceSP srcDevice,
     }
 
     if (config->soften()) {
-        KisLsUtils::applyGaussian(bumpmapSelection, d.applyGaussianRect, config->soften());
+        KisLsUtils::applyGaussianWithTransaction(bumpmapSelection, d.applyGaussianRect, config->soften());
     }
 
 
@@ -416,19 +424,20 @@ void KisLsBevelEmbossFilter::applyBevelEmboss(KisPaintDeviceSP srcDevice,
     //selection->convertToQImage(0, QRect(0,0,300,300)).save("4_shadows_sel.png");
 
     {
-        KisPaintDeviceSP dstDevice = dst->getProjection("00_bevel_shadow", config->shadowBlendMode(), srcDevice);
+        KisPaintDeviceSP dstDevice = dst->getProjection("00_bevel_shadow",
+                                                        config->shadowBlendMode(),
+                                                        config->shadowOpacity(),
+                                                        QBitArray(),
+                                                        srcDevice);
 
         const KoColor fillColor(config->shadowColor(), dstDevice->colorSpace());
         const QRect &fillRect = d.shadowHighlightsFinalRect;
-        KisPaintDeviceSP fillDevice = new KisPaintDevice(dstDevice->colorSpace());
-        fillDevice->setDefaultPixel(fillColor);
-        KisPainter gc(dstDevice);
 
-        gc.setSelection(baseSelection);
-        gc.setCompositeOp(COMPOSITE_OVER);
-        env->setupFinalPainter(&gc, config->shadowOpacity(), QBitArray());
-        gc.bitBlt(fillRect.topLeft(), fillDevice, fillRect);
-        gc.end();
+        KisCachedPaintDevice::Guard d1(dstDevice, *env->cachedPaintDevice());
+        KisPaintDeviceSP fillDevice = d1.device();
+        fillDevice->setDefaultPixel(fillColor);
+
+        KisPainter::copyAreaOptimized(fillRect.topLeft(), fillDevice, dstDevice, fillRect, baseSelection);
     }
 
     selection->clear();
@@ -439,35 +448,39 @@ void KisLsBevelEmbossFilter::applyBevelEmboss(KisPaintDeviceSP srcDevice,
     //selection->convertToQImage(0, QRect(0,0,300,300)).save("5_highlights_sel.png");
 
     {
-        KisPaintDeviceSP dstDevice = dst->getProjection("01_bevel_highlight", config->highlightBlendMode(), srcDevice);
+        KisPaintDeviceSP dstDevice = dst->getProjection("01_bevel_highlight",
+                                                        config->highlightBlendMode(),
+                                                        config->highlightOpacity(),
+                                                        QBitArray(),
+                                                        srcDevice);
 
         const KoColor fillColor(config->highlightColor(), dstDevice->colorSpace());
         const QRect &fillRect = d.shadowHighlightsFinalRect;
-        KisPaintDeviceSP fillDevice = new KisPaintDevice(dstDevice->colorSpace());
+
+        KisCachedPaintDevice::Guard d1(dstDevice, *env->cachedPaintDevice());
+        KisPaintDeviceSP fillDevice = d1.device();
         fillDevice->setDefaultPixel(fillColor);
-        KisPainter gc(dstDevice);
-        gc.setSelection(baseSelection);
-        gc.setCompositeOp(COMPOSITE_OVER);
-        env->setupFinalPainter(&gc, config->highlightOpacity(), QBitArray());
-        gc.bitBlt(fillRect.topLeft(), fillDevice, fillRect);
-        gc.end();
+
+        KisPainter::copyAreaOptimized(fillRect.topLeft(), fillDevice, dstDevice, fillRect, baseSelection);
     }
 }
 
 void KisLsBevelEmbossFilter::processDirectly(KisPaintDeviceSP src,
                                              KisMultipleProjection *dst,
-                                         const QRect &applyRect,
-                                         KisPSDLayerStyleSP style,
-                                         KisLayerStyleFilterEnvironment *env) const
+                                             KisLayerStyleKnockoutBlower *blower,
+                                             const QRect &applyRect,
+                                             KisPSDLayerStyleSP style,
+                                             KisLayerStyleFilterEnvironment *env) const
 {
     Q_UNUSED(env);
+    Q_UNUSED(blower);
     KIS_ASSERT_RECOVER_RETURN(style);
 
     const psd_layer_effects_bevel_emboss *config = style->bevelAndEmboss();
     if (!KisLsUtils::checkEffectEnabled(config, dst)) return;
 
     KisLsUtils::LodWrapper<psd_layer_effects_bevel_emboss> w(env->currentLevelOfDetail(), config);
-    applyBevelEmboss(src, dst, applyRect, w.config, env);
+    applyBevelEmboss(src, dst, applyRect, w.config, style->resourcesInterface(), env);
 }
 
 QRect KisLsBevelEmbossFilter::neededRect(const QRect &rect, KisPSDLayerStyleSP style, KisLayerStyleFilterEnvironment *env) const
