@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2016 Boudewijn Rempt <boud@valdyas.org>
+ *  SPDX-FileCopyrightText: 2016 Boudewijn Rempt <boud@valdyas.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 #include "Document.h"
 #include <QPointer>
@@ -51,9 +39,10 @@
 #include <kis_filter_strategy.h>
 #include <kis_guides_config.h>
 #include <kis_coordinates_converter.h>
-#include <kis_time_range.h>
+#include <kis_time_span.h>
 #include <KisImportExportErrorCode.h>
-#include <KisPart.h>
+#include <kis_types.h>
+#include <kis_annotation.h>
 
 #include <KoColor.h>
 #include <KoColorSpace.h>
@@ -61,6 +50,7 @@
 #include <KoColorSpaceRegistry.h>
 #include <KoColorConversionTransformation.h>
 #include <KoDocumentInfo.h>
+#include <KisGlobalResourcesInterface.h>
 
 #include <InfoObject.h>
 #include <Node.h>
@@ -238,7 +228,7 @@ bool Document::setBackgroundColor(const QColor &color)
     KoColor background = KoColor(color, d->document->image()->colorSpace());
     d->document->image()->setDefaultProjectionColor(background);
 
-    d->document->image()->setModified();
+    d->document->image()->setModifiedWithoutUndo();
     d->document->image()->initialRefreshGraph();
 
     return true;
@@ -254,7 +244,7 @@ QString Document::documentInfo() const
 
 void Document::setDocumentInfo(const QString &document)
 {
-    KoXmlDocument doc;
+    QDomDocument doc;
     QString errorMsg;
     int errorLine, errorColumn;
     doc.setContent(document, &errorMsg, &errorLine, &errorColumn);
@@ -265,7 +255,7 @@ void Document::setDocumentInfo(const QString &document)
 QString Document::fileName() const
 {
     if (!d->document) return QString();
-    return d->document->url().toLocalFile();
+    return d->document->path();
 }
 
 void Document::setFileName(QString value)
@@ -273,7 +263,7 @@ void Document::setFileName(QString value)
     if (!d->document) return;
     QString mimeType = KisMimeDatabase::mimeTypeForFile(value, false);
     d->document->setMimeType(mimeType.toLatin1());
-    d->document->setUrl(QUrl::fromLocalFile(value));
+    d->document->setPath(value);
 }
 
 int Document::height() const
@@ -475,7 +465,7 @@ QByteArray Document::pixelData(int x, int y, int w, int h) const
 
 bool Document::close()
 {
-    bool retval = d->document->closeUrl(false);
+    bool retval = d->document->closePath(false);
 
     Q_FOREACH(KisView *view, KisPart::instance()->views()) {
         if (view->document() == d->document) {
@@ -486,7 +476,9 @@ bool Document::close()
     }
 
     KisPart::instance()->removeDocument(d->document, !d->ownsDocument);
+
     if (d->ownsDocument) {
+
         delete d->document;
     }
 
@@ -511,7 +503,7 @@ bool Document::exportImage(const QString &filename, const InfoObject &exportConf
     const QString outputFormatString = KisMimeDatabase::mimeTypeForFile(filename, false);
     const QByteArray outputFormat = outputFormatString.toLatin1();
 
-    return d->document->exportDocumentSync(QUrl::fromLocalFile(filename), outputFormat, exportConfiguration.configuration());
+    return d->document->exportDocumentSync(filename, outputFormat, exportConfiguration.configuration());
 }
 
 void Document::flatten()
@@ -574,7 +566,7 @@ void Document::shearImage(double angleX, double angleY)
 bool Document::save()
 {
     if (!d->document) return false;
-    if (d->document->url().isEmpty()) return false;
+    if (d->document->path().isEmpty()) return false;
 
     bool retval = d->document->save(true, 0);
     d->document->waitForSavingToComplete();
@@ -588,11 +580,11 @@ bool Document::saveAs(const QString &filename)
 
     const QString outputFormatString = KisMimeDatabase::mimeTypeForFile(filename, false);
     const QByteArray outputFormat = outputFormatString.toLatin1();
-    QUrl oldUrl = d->document->url();
-    d->document->setUrl(QUrl::fromLocalFile(filename));
-    bool retval = d->document->saveAs(QUrl::fromLocalFile(filename), outputFormat, true);
+    QString oldPath = d->document->path();
+    d->document->setPath(filename);
+    bool retval = d->document->saveAs(filename, outputFormat, true);
     d->document->waitForSavingToComplete();
-    d->document->setUrl(oldUrl);
+    d->document->setPath(oldPath);
 
     return retval;
 }
@@ -678,7 +670,7 @@ FillLayer *Document::createFillLayer(const QString &name, const QString generato
     KisGeneratorSP generator = KisGeneratorRegistry::instance()->value(generatorName);
     if (generator) {
 
-        KisFilterConfigurationSP config = generator->factoryConfiguration();
+        KisFilterConfigurationSP config = generator->factoryConfiguration(KisGlobalResourcesInterface::instance());
         Q_FOREACH(const QString property, configuration.properties().keys()) {
             config->setProperty(property, configuration.property(property));
         }
@@ -753,6 +745,15 @@ SelectionMask *Document::createSelectionMask(const QString &name)
     return new SelectionMask(image, name);
 }
 
+TransformMask *Document::createTransformMask(const QString &name)
+{
+    if (!d->document) return 0;
+    if (!d->document->image()) return 0;
+    KisImageSP image = d->document->image();
+
+    return new TransformMask(image, name);
+}
+
 QImage Document::projection(int x, int y, int w, int h) const
 {
     if (!d->document || !d->document->image()) return QImage();
@@ -996,7 +997,7 @@ void Document::setPlayBackRange(int start, int stop)
     if (!d->document) return;
     if (!d->document->image()) return;
 
-    const KisTimeRange newTimeRange = KisTimeRange(start, (stop-start));
+    const KisTimeSpan newTimeRange = KisTimeSpan::fromTimeWithDuration(start, (stop-start));
     d->document->image()->animationInterface()->setPlaybackRange(newTimeRange);
 }
 
@@ -1031,3 +1032,63 @@ void Document::setCurrentTime(int time)
 
     return d->document->image()->animationInterface()->requestTimeSwitchWithUndo(time);
 }
+
+QStringList Document::annotationTypes() const
+{
+    if (!d->document) return QStringList();
+
+    QStringList types;
+
+    KisImageSP image = d->document->image().toStrongRef();
+
+    if (!image) return QStringList();
+
+    vKisAnnotationSP_it beginIt = image->beginAnnotations();
+    vKisAnnotationSP_it endIt = image->endAnnotations();
+
+    vKisAnnotationSP_it it = beginIt;
+    while (it != endIt) {
+        if (!(*it) || (*it)->type().isEmpty()) {
+            qWarning() << "Warning: empty annotation";
+            it++;
+            continue;
+        }
+        types << (*it)->type();
+
+        it++;
+    }
+    return types;
+}
+
+QString Document::annotationDescription(const QString &type) const
+{
+    KisImageSP image = d->document->image().toStrongRef();
+    KisAnnotationSP annotation = image->annotation(type);
+    return annotation->description();
+}
+
+QByteArray Document::annotation(const QString &type)
+{
+    KisImageSP image = d->document->image().toStrongRef();
+    KisAnnotationSP annotation = image->annotation(type);
+    if (annotation) {
+        return annotation->annotation();
+    }
+    else {
+        return QByteArray();
+    }
+}
+
+void Document::setAnnotation(const QString &key, const QString &description, const QByteArray &annotation)
+{
+    KisAnnotation *a = new KisAnnotation(key, description, annotation);
+    KisImageSP image = d->document->image().toStrongRef();
+    image->addAnnotation(a);
+
+}
+
+void Document::removeAnnotation(const QString &type)
+{
+    KisImageSP image = d->document->image().toStrongRef();
+    image->removeAnnotation(type);
+}
