@@ -1,21 +1,8 @@
 /*
- * Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
- * Copyright (C) 2012 Boudewijn Rempt <boud@valdyas.org>
+ * SPDX-FileCopyrightText: 1998, 1999 Torben Weis <weis@kde.org>
+ * SPDX-FileCopyrightText: 2012 Boudewijn Rempt <boud@valdyas.org>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public License
- * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #include "KisApplication.h"
@@ -43,6 +30,9 @@
 #include <QSysInfo>
 #include <QTimer>
 #include <QWidget>
+#include <QImageReader>
+#include <QImageWriter>
+#include <QThread>
 
 #include <klocalizedstring.h>
 #include <kdesktopfile.h>
@@ -54,19 +44,15 @@
 #include <KoColorSpaceRegistry.h>
 #include <KoPluginLoader.h>
 #include <KoShapeRegistry.h>
-#include <KoDpi.h>
 #include "KoConfig.h"
-#include <resources/KoHashGeneratorProvider.h>
 #include <KoResourcePaths.h>
 #include <KisMimeDatabase.h>
 #include "thememanager.h"
-#include "KisPrintJob.h"
 #include "KisDocument.h"
 #include "KisMainWindow.h"
 #include "KisAutoSaveRecoveryDialog.h"
 #include "KisPart.h"
 #include <kis_icon.h>
-#include "kis_md5_generator.h"
 #include "kis_splash_screen.h"
 #include "kis_config.h"
 #include "flake/kis_shape_selection.h"
@@ -81,7 +67,7 @@
 #include "KisApplicationArguments.h"
 #include <kis_debug.h>
 #include "kis_action_registry.h"
-#include <kis_brush_server.h>
+#include <KoResourceServer.h>
 #include <KisResourceServerProvider.h>
 #include <KoResourceServerProvider.h>
 #include "kis_image_barrier_locker.h"
@@ -89,13 +75,29 @@
 #include "kis_spin_box_unit_manager.h"
 #include "kis_document_aware_spin_box_unit_manager.h"
 #include "KisViewManager.h"
-#include "kis_workspace_resource.h"
 #include <KisUsageLogger.h>
 
 #include <KritaVersionWrapper.h>
 #include <dialogs/KisSessionManagerDialog.h>
 
-#include "widgets/KisScreenColorPicker.h"
+#include <KisResourceCacheDb.h>
+#include <KisResourceLocator.h>
+#include <KisResourceLoader.h>
+#include <KisResourceLoaderRegistry.h>
+
+#include <kis_gbr_brush.h>
+#include <kis_png_brush.h>
+#include <kis_svg_brush.h>
+#include <kis_imagepipe_brush.h>
+#include <KoColorSet.h>
+#include <KoSegmentGradient.h>
+#include <KoStopGradient.h>
+#include <KoPattern.h>
+#include <kis_workspace_resource.h>
+#include <KisSessionResource.h>
+#include <resources/KoSvgSymbolCollectionResource.h>
+
+#include "widgets/KisScreenColorSampler.h"
 #include "KisDlgInternalColorSelector.h"
 
 #include <dialogs/KisAsyncAnimationFramesSaveDialog.h>
@@ -103,6 +105,10 @@
 #include "kis_file_layer.h"
 #include "kis_group_layer.h"
 #include "kis_node_commands_adapter.h"
+#include "KisSynchronizedConnection.h"
+#include <QThreadStorage>
+
+#include <kis_psd_layer_style.h>
 
 #include <config-seexpr.h>
 
@@ -110,6 +116,42 @@ namespace {
 const QTime appStartTime(QTime::currentTime());
 }
 
+namespace {
+struct AppRecursionInfo {
+    ~AppRecursionInfo() {
+        KIS_SAFE_ASSERT_RECOVER_NOOP(!eventRecursionCount);
+        KIS_SAFE_ASSERT_RECOVER_NOOP(postponedSynchronizationEvents.empty());
+    }
+
+    int eventRecursionCount {0};
+    std::queue<KisSynchronizedConnectionEvent> postponedSynchronizationEvents;
+};
+
+struct AppRecursionGuard {
+    AppRecursionGuard(AppRecursionInfo *info)
+        : m_info(info)
+    {
+        m_info->eventRecursionCount++;
+    }
+
+    ~AppRecursionGuard()
+    {
+        m_info->eventRecursionCount--;
+    }
+private:
+    AppRecursionInfo *m_info {0};
+};
+
+}
+
+/**
+ * We cannot make the recursion info be a part of KisApplication,
+ * because KisApplication also owns QQmlThread, which is destroyed
+ * after KisApplication::Private, which makes QThreadStorage spit
+ * a warning about being destroyed too early
+ */
+Q_GLOBAL_STATIC(QThreadStorage<AppRecursionInfo>, s_recursionInfo)
+
 class KisApplication::Private
 {
 public:
@@ -161,10 +203,10 @@ KisApplication::KisApplication(const QString &key, int &argc, char **argv)
 
     QString version = KritaVersionWrapper::versionString(true);
     setApplicationVersion(version);
-    setWindowIcon(KisIconUtils::loadIcon("krita"));
+    setWindowIcon(KisIconUtils::loadIcon("krita-branding"));
 
     if (qgetenv("KRITA_NO_STYLE_OVERRIDE").isEmpty()) {
-        QStringList styles = QStringList() << "breeze" << "fusion" << "plastique";
+        QStringList styles = QStringList() << "breeze" << "fusion";
         if (!styles.contains(style()->objectName().toLower())) {
             Q_FOREACH (const QString & style, styles) {
                 if (!setStyle(style)) {
@@ -176,10 +218,21 @@ KisApplication::KisApplication(const QString &key, int &argc, char **argv)
                 }
             }
         }
+
+        // if style is set from config, try to load that
+        KisConfig cfg(true);
+        QString widgetStyleFromConfig = cfg.widgetStyle();
+        if(widgetStyleFromConfig != "") {
+            qApp->setStyle(widgetStyleFromConfig);
+        }
+
     }
     else {
         qDebug() << "Style override disabled, using" << style()->objectName();
     }
+
+    // store the style name
+    qApp->setProperty(currentUnderlyingStyleNameProperty, style()->objectName());
 }
 
 #if defined(Q_OS_WIN) && defined(ENV32BIT)
@@ -211,13 +264,43 @@ BOOL isWow64()
 
 void KisApplication::initializeGlobals(const KisApplicationArguments &args)
 {
-    int dpiX = args.dpiX();
-    int dpiY = args.dpiY();
-    if (dpiX > 0 && dpiY > 0) {
-        KoDpi::setDPI(dpiX, dpiY);
-    }
+    // There are no globals to initialize from the arguments now. There used
+    // to be the `dpi` argument, but it doesn't do anything anymore.
 }
 
+void KisApplication::addResourceTypes()
+{
+    // All Krita's resource types
+    KoResourcePaths::addResourceType("markers", "data", "/styles/");
+    KoResourcePaths::addResourceType("kis_pics", "data", "/pics/");
+    KoResourcePaths::addResourceType("kis_images", "data", "/images/");
+    KoResourcePaths::addResourceType("metadata_schema", "data", "/metadata/schemas/");
+    KoResourcePaths::addResourceType("gmic_definitions", "data", "/gmic/");
+    KoResourcePaths::addResourceType("kis_shortcuts", "data", "/shortcuts/");
+    KoResourcePaths::addResourceType("kis_actions", "data", "/actions");
+    KoResourcePaths::addResourceType("kis_actions", "data", "/pykrita");
+    KoResourcePaths::addResourceType("icc_profiles", "data", "/color/icc");
+    KoResourcePaths::addResourceType("icc_profiles", "data", "/profiles/");
+    KoResourcePaths::addResourceType(ResourceType::FilterEffects, "data", "/effects/");
+    KoResourcePaths::addResourceType("tags", "data", "/tags/");
+    KoResourcePaths::addResourceType("templates", "data", "/templates");
+    KoResourcePaths::addResourceType("pythonscripts", "data", "/pykrita");
+    KoResourcePaths::addResourceType("preset_icons", "data", "/preset_icons");
+#if defined HAVE_SEEXPR
+    KoResourcePaths::addResourceType(ResourceType::SeExprScripts, "data", "/seexpr_scripts/", true);
+#endif
+
+    // Make directories for all resources we can save, and tags
+    KoResourcePaths::saveLocation("data", "/asl/", true);
+    KoResourcePaths::saveLocation("data", "/input/", true);
+    KoResourcePaths::saveLocation("data", "/pykrita/", true);
+    KoResourcePaths::saveLocation("data", "/color-schemes/", true);
+    KoResourcePaths::saveLocation("data", "/preset_icons/", true);
+    KoResourcePaths::saveLocation("data", "/preset_icons/tool_icons/", true);
+    KoResourcePaths::saveLocation("data", "/preset_icons/emblem_icons/", true);
+}
+
+
 bool KisApplication::event(QEvent *event)
 {
 
@@ -231,116 +314,66 @@ bool KisApplication::event(QEvent *event)
     return QApplication::event(event);
 }
 
-void KisApplication::addResourceTypes()
+
+bool KisApplication::registerResources()
 {
-    //    qDebug() << "addResourceTypes();";
-    // All Krita's resource types
-    KoResourcePaths::addResourceType("markers", "data", "/styles/");
-    KoResourcePaths::addResourceType("kis_pics", "data", "/pics/");
-    KoResourcePaths::addResourceType("kis_images", "data", "/images/");
-    KoResourcePaths::addResourceType("metadata_schema", "data", "/metadata/schemas/");
-    KoResourcePaths::addResourceType("kis_brushes", "data", "/brushes/");
-    KoResourcePaths::addResourceType("kis_taskset", "data", "/taskset/");
-    KoResourcePaths::addResourceType("kis_taskset", "data", "/taskset/");
-    KoResourcePaths::addResourceType("gmic_definitions", "data", "/gmic/");
-    KoResourcePaths::addResourceType("kis_resourcebundles", "data", "/bundles/");
-    KoResourcePaths::addResourceType("kis_defaultpresets", "data", "/defaultpresets/");
-    KoResourcePaths::addResourceType("kis_paintoppresets", "data", "/paintoppresets/");
-    KoResourcePaths::addResourceType("kis_workspaces", "data", "/workspaces/");
-    KoResourcePaths::addResourceType("kis_windowlayouts", "data", "/windowlayouts/");
-    KoResourcePaths::addResourceType("kis_sessions", "data", "/sessions/");
-    KoResourcePaths::addResourceType("psd_layer_style_collections", "data", "/asl");
-    KoResourcePaths::addResourceType("ko_patterns", "data", "/patterns/", true);
-    KoResourcePaths::addResourceType("ko_gradients", "data", "/gradients/");
-    KoResourcePaths::addResourceType("ko_gradients", "data", "/gradients/", true);
-    KoResourcePaths::addResourceType("ko_palettes", "data", "/palettes/", true);
-    KoResourcePaths::addResourceType("kis_shortcuts", "data", "/shortcuts/");
-    KoResourcePaths::addResourceType("kis_actions", "data", "/actions");
-    KoResourcePaths::addResourceType("kis_actions", "data", "/pykrita");
-    KoResourcePaths::addResourceType("icc_profiles", "data", "/color/icc");
-    KoResourcePaths::addResourceType("icc_profiles", "data", "/profiles/");
-    KoResourcePaths::addResourceType("ko_effects", "data", "/effects/");
-    KoResourcePaths::addResourceType("tags", "data", "/tags/");
-    KoResourcePaths::addResourceType("templates", "data", "/templates");
-    KoResourcePaths::addResourceType("pythonscripts", "data", "/pykrita");
-    KoResourcePaths::addResourceType("symbols", "data", "/symbols");
-    KoResourcePaths::addResourceType("preset_icons", "data", "/preset_icons");
-    KoResourcePaths::addResourceType("ko_gamutmasks", "data", "/gamutmasks/", true);
+    KisResourceLoaderRegistry *reg = KisResourceLoaderRegistry::instance();
+
+    reg->add(new KisResourceLoader<KisPaintOpPreset>(ResourceSubType::KritaPaintOpPresets, ResourceType::PaintOpPresets, i18n("Brush presets"),
+                                                     QStringList() << "application/x-krita-paintoppreset"));
+
+    reg->add(new KisResourceLoader<KisGbrBrush>(ResourceSubType::GbrBrushes, ResourceType::Brushes, i18n("Brush tips"), QStringList() << "image/x-gimp-brush"));
+    reg->add(new KisResourceLoader<KisImagePipeBrush>(ResourceSubType::GihBrushes, ResourceType::Brushes, i18n("Brush tips"), QStringList() << "image/x-gimp-brush-animated"));
+    reg->add(new KisResourceLoader<KisSvgBrush>(ResourceSubType::SvgBrushes, ResourceType::Brushes, i18n("Brush tips"), QStringList() << "image/svg+xml"));
+    reg->add(new KisResourceLoader<KisPngBrush>(ResourceSubType::PngBrushes, ResourceType::Brushes, i18n("Brush tips"), QStringList() << "image/png"));
+
+    reg->add(new KisResourceLoader<KoSegmentGradient>(ResourceSubType::SegmentedGradients, ResourceType::Gradients, i18n("Gradients"), QStringList() << "application/x-gimp-gradient"));
+    reg->add(new KisResourceLoader<KoStopGradient>(ResourceSubType::StopGradients, ResourceType::Gradients, i18n("Gradients"), QStringList() << "image/svg+xml"));
+
+    reg->add(new KisResourceLoader<KoColorSet>(ResourceType::Palettes, ResourceType::Palettes, i18n("Palettes"),
+                                     QStringList() << KisMimeDatabase::mimeTypeForSuffix("kpl")
+                                               << KisMimeDatabase::mimeTypeForSuffix("gpl")
+                                               << KisMimeDatabase::mimeTypeForSuffix("pal")
+                                               << KisMimeDatabase::mimeTypeForSuffix("act")
+                                               << KisMimeDatabase::mimeTypeForSuffix("aco")
+                                               << KisMimeDatabase::mimeTypeForSuffix("css")
+                                               << KisMimeDatabase::mimeTypeForSuffix("colors")
+                                               << KisMimeDatabase::mimeTypeForSuffix("xml")
+                                               << KisMimeDatabase::mimeTypeForSuffix("sbz")));
+
+
+    reg->add(new KisResourceLoader<KoPattern>(ResourceType::Patterns, ResourceType::Patterns, i18n("Patterns"), {"application/x-gimp-pattern", "image/x-gimp-pat", "application/x-gimp-pattern", "image/bmp", "image/jpeg", "image/png", "image/tiff"}));
+    reg->add(new KisResourceLoader<KisWorkspaceResource>(ResourceType::Workspaces, ResourceType::Workspaces, i18n("Workspaces"), QStringList() << "application/x-krita-workspace"));
+    reg->add(new KisResourceLoader<KoSvgSymbolCollectionResource>(ResourceType::Symbols, ResourceType::Symbols, i18n("SVG symbol libraries"), QStringList() << "image/svg+xml"));
+    reg->add(new KisResourceLoader<KisWindowLayoutResource>(ResourceType::WindowLayouts, ResourceType::WindowLayouts, i18n("Window layouts"), QStringList() << "application/x-krita-windowlayout"));
+    reg->add(new KisResourceLoader<KisSessionResource>(ResourceType::Sessions, ResourceType::Sessions, i18n("Sessions"), QStringList() << "application/x-krita-session"));
+    reg->add(new KisResourceLoader<KoGamutMask>(ResourceType::GamutMasks, ResourceType::GamutMasks, i18n("Gamut masks"), QStringList() << "application/x-krita-gamutmasks"));
 #if defined HAVE_SEEXPR
-    KoResourcePaths::addResourceType("kis_seexpr_scripts", "data", "/seexpr_scripts/", true);
+    reg->add(new KisResourceLoader<KisSeExprScript>(ResourceType::SeExprScripts, ResourceType::SeExprScripts, i18n("SeExpr Scripts"), QStringList() << "application/x-krita-seexpr-script"));
 #endif
+    // XXX: this covers only individual styles, not the library itself!
+    reg->add(new KisResourceLoader<KisPSDLayerStyle>(ResourceType::LayerStyles,
+                                                     ResourceType::LayerStyles,
+                                                     i18nc("Resource type name", "Layer styles"),
+                                                     QStringList() << "application/x-photoshop-style"));
 
-    //    // Extra directories to look for create resources. (Does anyone actually use that anymore?)
-    //    KoResourcePaths::addResourceDir("ko_gradients", "/usr/share/create/gradients/gimp");
-    //    KoResourcePaths::addResourceDir("ko_gradients", QDir::homePath() + QString("/.create/gradients/gimp"));
-    //    KoResourcePaths::addResourceDir("ko_patterns", "/usr/share/create/patterns/gimp");
-    //    KoResourcePaths::addResourceDir("ko_patterns", QDir::homePath() + QString("/.create/patterns/gimp"));
-    //    KoResourcePaths::addResourceDir("kis_brushes", "/usr/share/create/brushes/gimp");
-    //    KoResourcePaths::addResourceDir("kis_brushes", QDir::homePath() + QString("/.create/brushes/gimp"));
-    //    KoResourcePaths::addResourceDir("ko_palettes", "/usr/share/create/swatches");
-    //    KoResourcePaths::addResourceDir("ko_palettes", QDir::homePath() + QString("/.create/swatches"));
+    KConfigGroup cfg(KSharedConfig::openConfig(), "");
+    QString databaseLocation = cfg.readEntry(KisResourceCacheDb::dbLocationKey, "");
+    if (databaseLocation.isEmpty()) {
+        databaseLocation = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
+    }
 
-    // Make directories for all resources we can save, and tags
-    QDir d;
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/tags/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/asl/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/bundles/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/brushes/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/gradients/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/paintoppresets/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/palettes/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/patterns/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/taskset/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/workspaces/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/input/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/pykrita/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/symbols/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/color-schemes/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/preset_icons/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/preset_icons/tool_icons/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/preset_icons/emblem_icons/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/gamutmasks/");
-#if defined HAVE_SEEXPR
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/seexpr_scripts/");
-#endif
-}
+    if (!KisResourceCacheDb::initialize(databaseLocation)) {
+        QMessageBox::critical(qApp->activeWindow(), i18nc("@title:window", "Krita: Fatal error"), i18n("%1\n\nKrita will quit now.", KisResourceCacheDb::lastError()));
+    }
 
-void KisApplication::loadResources()
-{
-    //    qDebug() << "loadResources();";
-
-    setSplashScreenLoadingText(i18n("Loading Resources..."));
-    processEvents();
-    KoResourceServerProvider::instance();
-
-    setSplashScreenLoadingText(i18n("Loading Brush Presets..."));
-    processEvents();
-    KisResourceServerProvider::instance();
-
-    setSplashScreenLoadingText(i18n("Loading Brushes..."));
-    processEvents();
-    KisBrushServer::instance()->brushServer();
-
-    setSplashScreenLoadingText(i18n("Loading Bundles..."));
-    processEvents();
-    KisResourceBundleServerProvider::instance();
-}
-
-void KisApplication::loadResourceTags()
-{
-    //    qDebug() << "loadResourceTags()";
-
-    KoResourceServerProvider::instance()->patternServer()->loadTags();
-    KoResourceServerProvider::instance()->gradientServer()->loadTags();
-    KoResourceServerProvider::instance()->paletteServer()->loadTags();
-    KoResourceServerProvider::instance()->svgSymbolCollectionServer()->loadTags();
-    KisBrushServer::instance()->brushServer()->loadTags();
-    KisResourceServerProvider::instance()->workspaceServer()->loadTags();
-    KisResourceServerProvider::instance()->layerStyleCollectionServer()->loadTags();
-    KisResourceBundleServerProvider::instance()->resourceBundleServer()->loadTags();
-    KisResourceServerProvider::instance()->paintOpPresetServer()->loadTags();
-
-    KisResourceServerProvider::instance()->paintOpPresetServer()->clearOldSystemTags();
+    KisResourceLocator::LocatorError r = KisResourceLocator::instance()->initialize(KoResourcePaths::getApplicationRoot() + "/share/krita");
+    connect(KisResourceLocator::instance(), SIGNAL(progressMessage(const QString&)), this, SLOT(setSplashScreenLoadingText(const QString&)));
+    if (r != KisResourceLocator::LocatorError::Ok && qApp->inherits("KisApplication")) {
+        QMessageBox::critical(qApp->activeWindow(), i18nc("@title:window", "Krita: Fatal error"), KisResourceLocator::instance()->errorMessages().join('\n') + i18n("\n\nKrita will quit now."));
+        return false;
+    }
+    return true;
 }
 
 void KisApplication::loadPlugins()
@@ -375,7 +408,7 @@ bool KisApplication::start(const KisApplicationArguments &args)
 #ifdef ENV32BIT
 
     if (isWow64() && !cfg.readEntry("WarnedAbout32Bits", false)) {
-        QMessageBox::information(0,
+        QMessageBox::information(qApp->activeWindow(),
                                  i18nc("@title:window", "Krita: Warning"),
                                  i18n("You are running a 32 bits build on a 64 bits Windows.\n"
                                       "This is not recommended.\n"
@@ -416,8 +449,6 @@ bool KisApplication::start(const KisApplicationArguments &args)
         processEvents();
     }
 
-    KoHashGeneratorProvider::instance()->setGenerator("MD5", new KisMD5Generator());
-
     KConfigGroup group(KSharedConfig::openConfig(), "theme");
     Digikam::ThemeManager themeManager;
     themeManager.setCurrentTheme(group.readEntry("Theme", "Krita dark"));
@@ -435,10 +466,9 @@ bool KisApplication::start(const KisApplicationArguments &args)
     loadPlugins();
 
     // Load all resources
-    loadResources();
-
-    // Load all the tags
-    loadResourceTags();
+    if (!registerResources()) {
+        return false;
+    }
 
     // Load the gui plugins
     loadGuiPlugins();
@@ -472,7 +502,7 @@ bool KisApplication::start(const KisApplicationArguments &args)
 
         if (!args.windowLayout().isEmpty()) {
             KoResourceServer<KisWindowLayoutResource> * rserver = KisResourceServerProvider::instance()->windowLayoutServer();
-            KisWindowLayoutResource* windowLayout = rserver->resourceByName(args.windowLayout());
+            KisWindowLayoutResourceSP windowLayout = rserver->resource("", "", args.windowLayout());
             if (windowLayout) {
                 windowLayout->applyLayout();
             }
@@ -483,7 +513,7 @@ bool KisApplication::start(const KisApplicationArguments &args)
 
             if (!args.workspace().isEmpty()) {
                 KoResourceServer<KisWorkspaceResource> * rserver = KisResourceServerProvider::instance()->workspaceServer();
-                KisWorkspaceResource* workspace = rserver->resourceByName(args.workspace());
+                KisWorkspaceResourceSP workspace = rserver->resource("", "", args.workspace());
                 if (workspace) {
                     d->mainWindow->restoreWorkspace(workspace);
                 }
@@ -551,7 +581,7 @@ bool KisApplication::start(const KisApplicationArguments &args)
 
                     KisDocument *doc = kisPart->createDocument();
                     doc->setFileBatchMode(d->batchRun);
-                    bool result = doc->openUrl(QUrl::fromLocalFile(fileName));
+                    bool result = doc->openPath(fileName);
 
                     if (!result) {
                         errKrita << "Could not load " << fileName << ":" << doc->errorMessage();
@@ -566,10 +596,11 @@ bool KisApplication::start(const KisApplicationArguments &args)
                     }
 
                     qApp->processEvents(); // For vector layers to be updated
-                    doc->image()->waitForDone();
 
                     doc->setFileBatchMode(true);
-                    if (!doc->exportDocumentSync(QUrl::fromLocalFile(exportFileName), outputMimetype.toLatin1())) {
+                    doc->image()->waitForDone();
+
+                    if (!doc->exportDocumentSync(exportFileName, outputMimetype.toLatin1())) {
                         errKrita << "Could not export " << fileName << "to" << exportFileName << ":" << doc->errorMessage();
                     }
                     QTimer::singleShot(0, this, SLOT(quit()));
@@ -578,7 +609,7 @@ bool KisApplication::start(const KisApplicationArguments &args)
                 else if (exportSequence) {
                     KisDocument *doc = kisPart->createDocument();
                     doc->setFileBatchMode(d->batchRun);
-                    doc->openUrl(QUrl::fromLocalFile(fileName));
+                    doc->openPath(fileName);
                     qApp->processEvents(); // For vector layers to be updated
                     
                     if (!doc->image()->animationInterface()->hasAnimation()) {
@@ -599,7 +630,7 @@ bool KisApplication::start(const KisApplicationArguments &args)
                     exporter.setBatchMode(d->batchRun);
                     KisAsyncAnimationFramesSaveDialog::Result result =
                         exporter.regenerateRange(0);
-                    if (result == KisAsyncAnimationFramesSaveDialog::RenderFailed) {
+                    if (result != KisAsyncAnimationFramesSaveDialog::RenderComplete) {
                         errKrita << i18n("Failed to render animation frames!") << endl;
                     }
                     QTimer::singleShot(0, this, SLOT(quit()));
@@ -612,7 +643,7 @@ bool KisApplication::start(const KisApplicationArguments &args)
                     else {
                         KisMainWindow::OpenFlags flags = d->batchRun ? KisMainWindow::BatchMode : KisMainWindow::None;
 
-                        if (d->mainWindow->openDocument(QUrl::fromLocalFile(fileName), flags)) {
+                        if (d->mainWindow->openDocument(fileName, flags)) {
                             // Normal case, success
                             numberOfOpenDocuments++;
                         }
@@ -635,18 +666,18 @@ bool KisApplication::start(const KisApplicationArguments &args)
                                     d->mainWindow->viewManager()->activeNode());
             }
             else{
-                QMessageBox::warning(nullptr, i18nc("@title:window", "Krita:Warning"),
+                QMessageBox::warning(qApp->activeWindow(), i18nc("@title:window", "Krita:Warning"),
                                             i18n("Cannot add %1 as a file layer: the file does not exist.", fileLayer->path()));
             }
         }
         else if (this->isRunning()){
-            QMessageBox::warning(nullptr, i18nc("@title:window", "Krita:Warning"),
+            QMessageBox::warning(qApp->activeWindow(), i18nc("@title:window", "Krita:Warning"),
                                 i18n("Cannot add the file layer: no document is open.\n\n"
 "You can create a new document using the --new-image option, or you can open an existing file.\n\n"
 "If you instead want to add the file layer to a document in an already running instance of Krita, check the \"Allow only one instance of Krita\" checkbox in the settings (Settings -> General -> Window)."));
         }
         else {
-            QMessageBox::warning(nullptr, i18nc("@title:window", "Krita: Warning"),
+            QMessageBox::warning(qApp->activeWindow(), i18nc("@title:window", "Krita: Warning"),
                                 i18n("Cannot add the file layer: no document is open.\n"
                                      "You can either create a new file using the --new-image option, or you can open an existing file."));
         }
@@ -669,7 +700,7 @@ bool KisApplication::start(const KisApplicationArguments &args)
     if (!d->earlyFileOpenEvents.isEmpty()) {
         hideSplashScreen();
         Q_FOREACH(QString fileName, d->earlyFileOpenEvents) {
-            d->mainWindow->openDocument(QUrl::fromLocalFile(fileName), 0);
+            d->mainWindow->openDocument(fileName, QFlags<KisMainWindow::OpenFlag>());
         }
     }
 
@@ -679,6 +710,9 @@ bool KisApplication::start(const KisApplicationArguments &args)
 
 KisApplication::~KisApplication()
 {
+    if (!isRunning()) {
+        KisResourceCacheDb::deleteTemporaryResources();
+    }
 }
 
 void KisApplication::setSplashScreen(QWidget *splashScreen)
@@ -686,10 +720,9 @@ void KisApplication::setSplashScreen(QWidget *splashScreen)
     d->splashScreen = qobject_cast<KisSplashScreen*>(splashScreen);
 }
 
-void KisApplication::setSplashScreenLoadingText(QString textToLoad)
+void KisApplication::setSplashScreenLoadingText(const QString &textToLoad)
 {
     if (d->splashScreen) {
-        //d->splashScreen->loadingLabel->setText(textToLoad);
         d->splashScreen->setLoadingText(textToLoad);
         d->splashScreen->repaint();
     }
@@ -707,7 +740,55 @@ void KisApplication::hideSplashScreen()
 bool KisApplication::notify(QObject *receiver, QEvent *event)
 {
     try {
-        return QApplication::notify(receiver, event);
+        bool result = true;
+
+        /**
+         * KisApplication::notify() is called for every event loop processed in
+         * any thread, so we need to make sure our counters and postponed events
+         * queues are stored in a per-thread way.
+         */
+        AppRecursionInfo &info = s_recursionInfo->localData();
+
+        {
+            // QApplication::notify() can throw, so use RAII for counters
+            AppRecursionGuard guard(&info);
+
+            if (event->type() == KisSynchronizedConnectionBase::eventType()) {
+
+                if (info.eventRecursionCount > 1) {
+                    KisSynchronizedConnectionEvent *typedEvent = static_cast<KisSynchronizedConnectionEvent*>(event);
+                    KIS_SAFE_ASSERT_RECOVER_NOOP(typedEvent->destination == receiver);
+
+                    info.postponedSynchronizationEvents.emplace(KisSynchronizedConnectionEvent(*typedEvent));
+                } else {
+                    result = QApplication::notify(receiver, event);
+                }
+            } else {
+                result = QApplication::notify(receiver, event);
+            }
+        }
+
+        if (!info.eventRecursionCount) {
+            while (!info.postponedSynchronizationEvents.empty()) {
+                // QApplication::notify() can throw, so use RAII for counters
+                AppRecursionGuard guard(&info);
+
+                /// We must pop event from the queue **before** we call
+                /// QApplication::notify(), because it can throw!
+                KisSynchronizedConnectionEvent typedEvent = info.postponedSynchronizationEvents.front();
+                info.postponedSynchronizationEvents.pop();
+
+                if (!typedEvent.destination) {
+                    qWarning() << "WARNING: the destination object of KisSynchronizedConnection has been destroyed during postponed delivery";
+                    continue;
+                }
+
+                QApplication::notify(typedEvent.destination, &typedEvent);
+            }
+        }
+
+        return result;
+
     } catch (std::exception &e) {
         qWarning("Error %s sending event %i to object %s",
                  e.what(), event->type(), qPrintable(receiver->objectName()));
@@ -745,7 +826,7 @@ void KisApplication::executeRemoteArguments(QByteArray message, KisMainWindow *m
             }
             else if (QFile(filename).exists()) {
                 KisMainWindow::OpenFlags flags = d->batchRun ? KisMainWindow::BatchMode : KisMainWindow::None;
-                documentCreated |= mainWindow->openDocument(QUrl::fromLocalFile(filename), flags);
+                documentCreated |= mainWindow->openDocument(filename, flags);
             }
         }
     }
@@ -806,7 +887,7 @@ void KisApplication::fileOpenRequested(const QString &url)
     }
 
     KisMainWindow::OpenFlags flags = d->batchRun ? KisMainWindow::BatchMode : KisMainWindow::None;
-    d->mainWindow->openDocument(QUrl::fromLocalFile(url), flags);
+    d->mainWindow->openDocument(url, flags);
 }
 
 
@@ -856,15 +937,15 @@ void KisApplication::checkAutosaveFiles()
         }
 
         if (autosaveFiles.size() > 0) {
-            QList<QUrl> autosaveUrls;
+            QList<QString> autosavePaths;
             Q_FOREACH (const QString &autoSaveFile, autosaveFiles) {
-                const QUrl url = QUrl::fromLocalFile(dir.absolutePath() + QLatin1Char('/') + autoSaveFile);
-                autosaveUrls << url;
+                const QString path = dir.absolutePath() + QLatin1Char('/') + autoSaveFile;
+                autosavePaths << path;
             }
             if (d->mainWindow) {
-                Q_FOREACH (const QUrl &url, autosaveUrls) {
+                Q_FOREACH (const QString &path, autosavePaths) {
                     KisMainWindow::OpenFlags flags = d->batchRun ? KisMainWindow::BatchMode : KisMainWindow::None;
-                    d->mainWindow->openDocument(url, flags | KisMainWindow::RecoveryFile);
+                    d->mainWindow->openDocument(path, flags | KisMainWindow::RecoveryFile);
                 }
             }
         }
@@ -878,9 +959,8 @@ bool KisApplication::createNewDocFromTemplate(const QString &fileName, KisMainWi
 {
     QString templatePath;
 
-    const QUrl templateUrl = QUrl::fromLocalFile(fileName);
     if (QFile::exists(fileName)) {
-        templatePath = templateUrl.toLocalFile();
+        templatePath = fileName;
         dbgUI << "using full path...";
     }
     else {
@@ -893,10 +973,10 @@ bool KisApplication::createNewDocFromTemplate(const QString &fileName, KisMainWi
         }
 
         if (paths.isEmpty()) {
-            QMessageBox::critical(0, i18nc("@title:window", "Krita"),
+            QMessageBox::critical(qApp->activeWindow(), i18nc("@title:window", "Krita"),
                                   i18n("No template found for: %1", desktopName));
         } else if (paths.count() > 1) {
-            QMessageBox::critical(0, i18nc("@title:window", "Krita"),
+            QMessageBox::critical(qApp->activeWindow(), i18nc("@title:window", "Krita"),
                                   i18n("Too many templates found for: %1", desktopName));
         } else {
             templatePath = paths.at(0);
@@ -904,25 +984,16 @@ bool KisApplication::createNewDocFromTemplate(const QString &fileName, KisMainWi
     }
 
     if (!templatePath.isEmpty()) {
-        QUrl templateBase;
-        templateBase.setPath(templatePath);
         KDesktopFile templateInfo(templatePath);
 
-        QString templateName = templateInfo.readUrl();
-        QUrl templateURL;
-        templateURL.setPath(templateBase.adjusted(QUrl::RemoveFilename|QUrl::StripTrailingSlash).path() + '/' + templateName);
-        if (templateURL.scheme().isEmpty()) {
-            templateURL.setScheme("file");
-        }
-
         KisMainWindow::OpenFlags batchFlags = d->batchRun ? KisMainWindow::BatchMode : KisMainWindow::None;
-        if (mainWindow->openDocument(templateURL, KisMainWindow::Import | batchFlags)) {
+        if (mainWindow->openDocument(templatePath, KisMainWindow::Import | batchFlags)) {
             dbgUI << "Template loaded...";
             return true;
         }
         else {
-            QMessageBox::critical(0, i18nc("@title:window", "Krita"),
-                                  i18n("Template %1 failed to load.", templateURL.toDisplayString()));
+            QMessageBox::critical(qApp->activeWindow(), i18nc("@title:window", "Krita"),
+                                  i18n("Template %1 failed to load.", fileName));
         }
     }
 
@@ -952,7 +1023,7 @@ void KisApplication::resetConfig()
                 backupKritarcFile.remove();
             }
 
-            QMessageBox::information(0,
+            QMessageBox::information(qApp->activeWindow(),
                                  i18nc("@title:window", "Krita"),
                                  i18n("Krita configurations reset!\n\n"
                                       "Backup file was created at: %1\n\n"
@@ -966,7 +1037,7 @@ void KisApplication::resetConfig()
             kritarcFile.close();
         }
         else {
-            QMessageBox::warning(0,
+            QMessageBox::warning(qApp->activeWindow(),
                                  i18nc("@title:window", "Krita"),
                                  i18n("Failed to clear %1\n\n"
                                       "Please make sure no other program is using the file and try again.",
@@ -984,17 +1055,17 @@ void KisApplication::resetConfig()
     KConfigGroup cfg = KSharedConfig::openConfig()->group("MainWindow");
 
     QString currentWorkspace = cfg.readEntry<QString>("CurrentWorkspace", "Default");
-    KoResourceServer<KisWorkspaceResource> *rserver = KisResourceServerProvider::instance()->workspaceServer();
-    KisWorkspaceResource *workspace = rserver->resourceByName(currentWorkspace);
+    KoResourceServer<KisWorkspaceResource> * rserver = KisResourceServerProvider::instance()->workspaceServer();
+    KisWorkspaceResourceSP workspace = rserver->resource("", "", currentWorkspace);
 
     if (workspace) {
         d->mainWindow->restoreWorkspace(workspace);
     }
 }
 
-void KisApplication::askresetConfig()
+void KisApplication::askResetConfig()
 {
-    bool ok = QMessageBox::question(0,
+    bool ok = QMessageBox::question(qApp->activeWindow(),
                                     i18nc("@title:window", "Krita"),
                                     i18n("Do you want to clear the settings file?"),
                                     QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::Yes;
