@@ -1,20 +1,7 @@
 /*
- * Copyright (c) 2015 Boudewijn Rempt <boud@valdyas.org>
+ * SPDX-FileCopyrightText: 2015 Boudewijn Rempt <boud@valdyas.org>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 #include "KisApplicationArguments.h"
 
@@ -98,33 +85,78 @@ KisApplicationArguments::KisApplicationArguments(const QApplication &app)
     parser.addOption(QCommandLineOption(QStringList() << QLatin1String("canvasonly"), i18n("Start Krita in canvas-only mode")));
     parser.addOption(QCommandLineOption(QStringList() << QLatin1String("nosplash"), i18n("Do not show the splash screen")));
     parser.addOption(QCommandLineOption(QStringList() << QLatin1String("fullscreen"), i18n("Start Krita in full-screen mode")));
-    parser.addOption(QCommandLineOption(QStringList() << QLatin1String("dpi"), i18n("Override display DPI"), QLatin1String("dpiX,dpiY")));
+    {
+        QCommandLineOption opt(QStringList() << QLatin1String("dpi"), i18n("Override display DPI"), QLatin1String("dpiX,dpiY"));
+        opt.setFlags(QCommandLineOption::HiddenFromHelp);
+        parser.addOption(opt);
+    }
     parser.addOption(QCommandLineOption(QStringList() << QLatin1String("export"), i18n("Export to the given filename and exit")));
     parser.addOption(QCommandLineOption(QStringList() << QLatin1String("export-sequence"), i18n("Export animation to the given filename and exit")));
     parser.addOption(QCommandLineOption(QStringList() << QLatin1String("export-filename"), i18n("Filename for export"), QLatin1String("filename")));
     parser.addOption(QCommandLineOption(QStringList() << QLatin1String("file-layer"), i18n("File layer to be added to existing or new file"), QLatin1String("file-layer")));
     parser.addPositionalArgument(QLatin1String("[file(s)]"), i18n("File(s) or URL(s) to open"));
 
+    QStringList filteredArgs;
+#ifdef Q_OS_WIN
+    {
+        auto checkIsIgnoreEpic = [](const QString &arg) {
+            // List according to https://dev.epicgames.com/docs/services/en-US/Interfaces/Auth/index.html#epicgameslauncher
+            static const QStringList epicIgnoreArgsStart = {
+                QStringLiteral("AUTH_PASSWORD="),
+                QStringLiteral("AUTH_TYPE="),
+                QStringLiteral("epicapp="),
+                QStringLiteral("epicenv="),
+                QStringLiteral("epicusername="),
+                QStringLiteral("epicuserid="),
+                QStringLiteral("epiclocale="),
+            };
+            static const QStringList epicIgnoreArgsExact = {
+                QStringLiteral("EpicPortal"),
+            };
+            QStringRef argDashless(&arg);
+            // Strip leading dashes.
+            while (argDashless.startsWith('-')) {
+                argDashless = argDashless.mid(1);
+            }
+            Q_FOREACH(const auto &argToIgnore, epicIgnoreArgsStart) {
+                if (argDashless.startsWith(argToIgnore, Qt::CaseInsensitive)) {
+                    return true;
+                }
+            }
+            Q_FOREACH(const auto &argToIgnore, epicIgnoreArgsExact) {
+                if (argDashless.compare(argToIgnore, Qt::CaseInsensitive) == 0) {
+                    return true;
+                }
+            }
+            return false;
+        };
 
-    if (parser.parse(app.arguments())) {
-        parser.process(app);
+        QStringListIterator iter(app.arguments());
+        if (iter.hasNext()) {
+            // First argument is application name.
+            filteredArgs.append(iter.next());
+
+            bool isAfterDoubleDash = false;
+            while (iter.hasNext()) {
+                QString arg = iter.next();
+                if (arg == QLatin1String("--")) {
+                    isAfterDoubleDash = true;
+                }
+                if (isAfterDoubleDash || !checkIsIgnoreEpic(arg)) {
+                    filteredArgs.append(arg);
+                }
+            }
+        }
     }
+#else // !defined(Q_OS_WIN)
+    filteredArgs = app.arguments();
+#endif // Q_OS_WIN
+
+    parser.process(filteredArgs);
 
     QString dpiValues = parser.value("dpi");
     if (!dpiValues.isEmpty()) {
-        int sep = dpiValues.indexOf(QRegExp("[x, ]"));
-        bool ok = true;
-        if (sep != -1) {
-            d->dpiY = dpiValues.mid(sep + 1).toInt(&ok);
-            dpiValues.truncate(sep);
-        }
-        if (ok) {
-            d->dpiX = dpiValues.toInt(&ok);
-            if (ok) {
-                if (!d->dpiY)
-                    d->dpiY = d->dpiX;
-            }
-        }
+        qWarning() << "The `dpi` argument does nothing!";
     }
 
     QString newImageValues = parser.value("new-image");
@@ -163,8 +195,8 @@ KisApplicationArguments::KisApplicationArguments(const KisApplicationArguments &
     : d(new Private)
 {
     d->filenames = rhs.filenames();
-    d->dpiX = rhs.dpiX();
-    d->dpiY = rhs.dpiY();
+    d->dpiX = rhs.d->dpiX;
+    d->dpiY = rhs.d->dpiY;
     d->doTemplate = rhs.doTemplate();
     d->exportAs = rhs.exportAs();
     d->exportFileName = rhs.exportFileName();
@@ -179,8 +211,8 @@ KisApplicationArguments::KisApplicationArguments(const KisApplicationArguments &
 void KisApplicationArguments::operator=(const KisApplicationArguments &rhs)
 {
     d->filenames = rhs.filenames();
-    d->dpiX = rhs.dpiX();
-    d->dpiY = rhs.dpiY();
+    d->dpiX = rhs.d->dpiX;
+    d->dpiY = rhs.d->dpiY;
     d->doTemplate = rhs.doTemplate();
     d->exportAs = rhs.exportAs();
     d->exportFileName = rhs.exportFileName();
@@ -194,8 +226,7 @@ void KisApplicationArguments::operator=(const KisApplicationArguments &rhs)
 
 QByteArray KisApplicationArguments::serialize()
 {
-    QByteArray ba;
-    QBuffer buf(&ba);
+    QBuffer buf;
     buf.open(QIODevice::WriteOnly);
     QDataStream ds(&buf);
     ds.setVersion(QDataStream::Qt_5_0);
@@ -224,7 +255,7 @@ QByteArray KisApplicationArguments::serialize()
 
     buf.close();
 
-    return ba;
+    return buf.data();
 }
 
 KisApplicationArguments KisApplicationArguments::deserialize(QByteArray &serialized)
@@ -271,16 +302,6 @@ QStringList KisApplicationArguments::filenames() const
     return d->filenames;
 }
 
-int KisApplicationArguments::dpiX() const
-{
-    return d->dpiX;
-}
-
-int KisApplicationArguments::dpiY() const
-{
-    return d->dpiY;
-}
-
 bool KisApplicationArguments::doTemplate() const
 {
     return d->doTemplate;
