@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2016 Boudewijn Rempt <boud@valdyas.org>
+ *  SPDX-FileCopyrightText: 2016 Boudewijn Rempt <boud@valdyas.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "KisDlgAnimationRenderer.h"
@@ -37,7 +25,7 @@
 #include <krita_container_utils.h>
 #include <kis_image.h>
 #include <kis_image_animation_interface.h>
-#include <kis_time_range.h>
+#include <kis_time_span.h>
 #include <KisImportExportManager.h>
 #include <kis_config_widget.h>
 #include <KisDocument.h>
@@ -49,7 +37,9 @@
 #include "kis_acyclic_signal_connector.h"
 #include "KisVideoSaver.h"
 #include "KisAnimationRenderingOptions.h"
+#include "animation/KisFFMpegWrapper.h"
 #include "VideoExportOptionsDialog.h"
+#include "kis_image_config.h"
 
 
 KisDlgAnimationRenderer::KisDlgAnimationRenderer(KisDocument *doc, QWidget *parent)
@@ -106,12 +96,20 @@ KisDlgAnimationRenderer::KisDlgAnimationRenderer(KisDocument *doc, QWidget *pare
         }
         m_page->cmbRenderType->addItem(description, mime);
     }
-
+    
+    
+    m_page->cmbScaleFilter->addItem(i18nc("bicubic filtering", "bicubic"), "bicubic");
+    m_page->cmbScaleFilter->addItem(i18nc("bilinear filtering", "bilinear"), "bilinear");
+    m_page->cmbScaleFilter->addItem(i18nc("lanczos3 filtering", "lanczos3"), "lanczos");
+    m_page->cmbScaleFilter->addItem(i18nc("nearest neighbor filtering", "neighbor"), "neighbor");
+    m_page->cmbScaleFilter->addItem(i18nc("spline filtering", "spline"), "spline");
+    
     m_page->videoFilename->setMode(KoFileDialog::SaveFile);
 
     m_page->ffmpegLocation->setMode(KoFileDialog::OpenFile);
 
     m_page->cmbRenderType->setCurrentIndex(cfg.readEntry<int>("AnimationRenderer/render_type", 0));
+    m_page->cmbRenderType->setCurrentIndex(cfg.readEntry<int>("AnimationRenderer/scale_mode", 0));
 
     connect(m_page->bnExportOptions, SIGNAL(clicked()), this, SLOT(sequenceMimeTypeOptionsClicked()));
     connect(m_page->bnRenderOptions, SIGNAL(clicked()), this, SLOT(selectRenderOptions()));
@@ -132,10 +130,9 @@ KisDlgAnimationRenderer::KisDlgAnimationRenderer(KisDocument *doc, QWidget *pare
     constrainsConnector->createCoordinatedConnector()->connectForwardInt(m_page->intHeight, SIGNAL(valueChanged(int)), this, SLOT(slotLockAspectRatioDimensionsHeight(int)));
 
     {
-        KisPropertiesConfigurationSP settings = cfg.exportConfiguration("ANIMATION_EXPORT");
+        KisPropertiesConfigurationSP settings = loadLastConfiguration("ANIMATION_EXPORT");
         KisAnimationRenderingOptions options;
         options.fromProperties(settings);
-
         initializeRenderSettings(*doc, options);
     }
 
@@ -181,6 +178,8 @@ QStringList KisDlgAnimationRenderer::makeVideoMimeTypesList()
     QStringList supportedMimeTypes = QStringList();
     supportedMimeTypes << "video/x-matroska";
     supportedMimeTypes << "image/gif";
+    supportedMimeTypes << "image/apng";    
+    supportedMimeTypes << "image/webp";       
     supportedMimeTypes << "video/ogg";
     supportedMimeTypes << "video/mp4";
     supportedMimeTypes << "video/webm";
@@ -193,6 +192,17 @@ bool KisDlgAnimationRenderer::imageMimeSupportsHDR(QString &mime)
     return (mime == "image/png");
 }
 
+KisPropertiesConfigurationSP KisDlgAnimationRenderer::loadLastConfiguration(QString configurationID) {
+    KisConfig globalConfig(true);
+    return globalConfig.exportConfiguration(configurationID);
+}
+
+void KisDlgAnimationRenderer::saveLastUsedConfiguration(QString configurationID, KisPropertiesConfigurationSP config)
+{
+    KisConfig globalConfig(false);
+    globalConfig.setExportConfiguration(configurationID, config);
+}
+
 void KisDlgAnimationRenderer::initializeRenderSettings(const KisDocument &doc, const KisAnimationRenderingOptions &lastUsedOptions)
 {
     const QString documentPath = m_doc->localFilePath();
@@ -238,6 +248,13 @@ void KisDlgAnimationRenderer::initializeRenderSettings(const KisDocument &doc, c
         }
     }
 
+    for (int i = 0; i < m_page->cmbScaleFilter->count(); ++i) {
+        if (m_page->cmbScaleFilter->itemData(i).toString() == lastUsedOptions.scaleFilter) {
+            m_page->cmbScaleFilter->setCurrentIndex(i);
+            break;
+        }
+    }    
+    
     m_page->chkOnlyUniqueFrames->setChecked(lastUsedOptions.wantsOnlyUniqueFrameSequence);
 
     if (lastUsedOptions.shouldDeleteSequence) {
@@ -252,8 +269,7 @@ void KisDlgAnimationRenderer::initializeRenderSettings(const KisDocument &doc, c
 
 
     {
-        KisConfig cfg(true);
-        KisPropertiesConfigurationSP settings = cfg.exportConfiguration("VIDEO_ENCODER");
+        KisPropertiesConfigurationSP settings = loadLastConfiguration("VIDEO_ENCODER");
 
         getDefaultVideoEncoderOptions(lastUsedOptions.videoMimeType, settings,
                                       &m_customFFMpegOptionsString,
@@ -261,23 +277,68 @@ void KisDlgAnimationRenderer::initializeRenderSettings(const KisDocument &doc, c
     }
 
     {
-        KisConfig cfg(true);
-        KisPropertiesConfigurationSP settings = cfg.exportConfiguration("img_sequence/" + lastUsedOptions.frameMimeType);
-
+        KisPropertiesConfigurationSP settings = loadLastConfiguration("img_sequence/" + lastUsedOptions.frameMimeType);
         m_wantsRenderWithHDR = settings->getPropertyLazy("saveAsHDR", m_wantsRenderWithHDR);
     }
 
     m_page->ffmpegLocation->setStartDir(QFileInfo(m_doc->localFilePath()).path());
-    m_page->ffmpegLocation->setFileName(findFFMpeg(lastUsedOptions.ffmpegPath));
+    KisConfig cfg(false);
+    QString ffmpegPath = cfg.ffmpegLocation();
+    QJsonObject ffmpegJsonObj = KisFFMpegWrapper::findFFMpeg(ffmpegPath.isEmpty() ? lastUsedOptions.ffmpegPath:ffmpegPath);
+
+    ffmpegPath = ffmpegJsonObj["path"].toString();
+    ffmpegVersion = ffmpegJsonObj["enabled"].toBool() ? ffmpegJsonObj["version"].toString():"None";
+
+    m_page->ffmpegLocation->setFileName(ffmpegPath);
+    cfg.setFFMpegLocation(ffmpegPath);
+    m_page->lblFFMpegVersion->setText("FFMpeg Version: " + ffmpegVersion + (ffmpegJsonObj["encoder"].toObject()["h264"].toBool() ? "":" (MP4/MKV UNSUPPORTED)"));
+    
+    ffmpegWarningCheck();
+
+    connect(m_page->ffmpegLocation, SIGNAL(textChanged(QString)), SLOT(slotFFMpegChanged(QString)));
 
     // Initialize these settings based on the current document context..
     m_page->intStart->setValue(doc.image()->animationInterface()->playbackRange().start());
     m_page->intEnd->setValue(doc.image()->animationInterface()->playbackRange().end());
     m_page->intFramesPerSecond->setValue(doc.image()->animationInterface()->framerate());
 
+    if (!doc.image()->animationInterface()->exportSequenceFilePath().isEmpty()
+        && QDir(doc.image()->animationInterface()->exportSequenceFilePath()).exists() ) {
+        m_page->dirRequester->setStartDir(doc.image()->animationInterface()->exportSequenceFilePath());
+        m_page->dirRequester->setFileName(doc.image()->animationInterface()->exportSequenceFilePath());
+    }
+
+    if (!doc.image()->animationInterface()->exportSequenceBaseName().isEmpty()) {
+        m_page->txtBasename->setText(doc.image()->animationInterface()->exportSequenceBaseName());
+    }
+
+    if (doc.image()->animationInterface()->exportInitialFrameNumber() != -1) {
+        m_page->sequenceStart->setValue(doc.image()->animationInterface()->exportInitialFrameNumber());
+    }
+
     m_page->chkIncludeAudio->setChecked(!doc.image()->animationInterface()->isAudioMuted());
 }
 
+void KisDlgAnimationRenderer::slotFFMpegChanged(const QString& path) {
+    KisConfig cfg(false);
+    QJsonObject ffmpegChangeJsonObj = KisFFMpegWrapper::findFFMpeg(path);
+    ffmpegVersion = ffmpegChangeJsonObj["enabled"].toBool() ? ffmpegChangeJsonObj["version"].toString():"None";
+
+    cfg.setFFMpegLocation(ffmpegChangeJsonObj["path"].toString());
+    m_page->lblFFMpegVersion->setText("FFMpeg Version: " + ffmpegVersion + (ffmpegChangeJsonObj["encoder"].toObject()["h264"].toBool() ? "":" (MP4/MKV UNSUPPORTED)"));
+
+    ffmpegWarningCheck();
+}
+
+void KisDlgAnimationRenderer::ffmpegWarningCheck() {
+    const QString mimeType = m_page->cmbRenderType->itemData(m_page->cmbRenderType->currentIndex()).toString();
+
+    QRegularExpression minVerFFMpegRX("^n{0,1}(?:[0-3]|4\\.[01])[\\.\\-]");
+    QRegularExpressionMatch minVerFFMpegMatch = minVerFFMpegRX.match(ffmpegVersion);
+
+    m_page->lblGifWarningFFMpeg->setVisible((mimeType == "image/gif" && minVerFFMpegMatch.hasMatch() ));
+}
+
 QString KisDlgAnimationRenderer::defaultVideoFileName(KisDocument *doc, const QString &mimeType)
 {
     const QString docFileName = !doc->localFilePath().isEmpty() ?
@@ -286,15 +347,17 @@ QString KisDlgAnimationRenderer::defaultVideoFileName(KisDocument *doc, const QS
     return
         QString("%1.%2")
             .arg(QFileInfo(docFileName).completeBaseName())
-            .arg(KisMimeDatabase::suffixesForMimeType(mimeType).first());
+            .arg(KisMimeDatabase::suffixesForMimeType( mimeType == "image/apng" ? "image/png":mimeType ).first());
 }
 
 void KisDlgAnimationRenderer::selectRenderType(int index)
 {
     const QString mimeType = m_page->cmbRenderType->itemData(index).toString();
 
-    m_page->bnRenderOptions->setEnabled(mimeType != "image/gif");
-    m_page->lblGifWarning->setVisible((mimeType == "image/gif" && m_page->intFramesPerSecond->value() > 50));
+    // m_page->bnRenderOptions->setEnabled(mimeType != "image/gif" && mimeType != "image/webp" && mimeType != "image/png" );
+    m_page->lblGifWarningFPS->setVisible((mimeType == "image/gif" && m_page->intFramesPerSecond->value() > 50));
+
+    ffmpegWarningCheck();
 
     QString videoFileName = defaultVideoFileName(m_doc, mimeType);
 
@@ -304,7 +367,7 @@ void KisDlgAnimationRenderer::selectRenderType(int index)
         const QString path = info.path();
 
         videoFileName =
-            QString("%1%2%3.%4").arg(path).arg('/').arg(baseName).arg(KisMimeDatabase::suffixesForMimeType(mimeType).first());
+            QString("%1%2%3.%4").arg(path).arg('/').arg(baseName).arg(KisMimeDatabase::suffixesForMimeType( mimeType == "image/apng" ? "image/png":mimeType ).first());
 
     }
     m_page->videoFilename->setMimeTypeFilters(QStringList() << mimeType, mimeType);
@@ -316,9 +379,7 @@ void KisDlgAnimationRenderer::selectRenderType(int index)
         // If this is removed from the configuration, ogg vorbis can fail to render on first attempt. BUG:421658
         // This should be revisited at some point, too much configuration juggling in this class makes it error-prone...
 
-        KisConfig cfg(true);
-        KisPropertiesConfigurationSP settings = cfg.exportConfiguration("VIDEO_ENCODER");
-
+        KisPropertiesConfigurationSP settings = loadLastConfiguration("VIDEO_ENCODER");
         getDefaultVideoEncoderOptions(mimeType, settings,
                                       &m_customFFMpegOptionsString,
                                       &m_wantsRenderWithHDR);
@@ -340,8 +401,7 @@ void KisDlgAnimationRenderer::selectRenderOptions()
     encoderConfigWidget->setSupportsHDR(true);
 
     {
-        KisConfig cfg(true);
-        KisPropertiesConfigurationSP settings = cfg.exportConfiguration("VIDEO_ENCODER");
+        KisPropertiesConfigurationSP settings = loadLastConfiguration("VIDEO_ENCODER");
         encoderConfigWidget->setConfiguration(settings);
         encoderConfigWidget->setHDRConfiguration(m_wantsRenderWithHDR);
     }
@@ -350,8 +410,7 @@ void KisDlgAnimationRenderer::selectRenderOptions()
     dlg.setMainWidget(encoderConfigWidget);
     dlg.setButtons(KoDialog::Ok | KoDialog::Cancel);
     if (dlg.exec() == QDialog::Accepted) {
-        KisConfig cfg(false);
-        cfg.setExportConfiguration("VIDEO_ENCODER", encoderConfigWidget->configuration());
+        saveLastUsedConfiguration("VIDEO_ENCODER", encoderConfigWidget->configuration());
         m_customFFMpegOptionsString = encoderConfigWidget->customUserOptionsString();
         m_wantsRenderWithHDR = encoderConfigWidget->videoConfiguredForHDR();
     }
@@ -372,9 +431,8 @@ void KisDlgAnimationRenderer::sequenceMimeTypeOptionsClicked()
         frameExportConfigWidget = filter->createConfigurationWidget(0, KisDocument::nativeFormatMimeType(), mimetype.toLatin1());
 
         if (frameExportConfigWidget) {
-            KisConfig cfg(true);
 
-            KisPropertiesConfigurationSP exportConfig = cfg.exportConfiguration("img_sequence/" + mimetype);
+            KisPropertiesConfigurationSP exportConfig = loadLastConfiguration("img_sequence/" + mimetype);
             if (exportConfig) {
                 KisImportExportManager::fillStaticExportConfigurationProperties(exportConfig, m_image);
             }
@@ -392,9 +450,8 @@ void KisDlgAnimationRenderer::sequenceMimeTypeOptionsClicked()
             dlg.setMainWidget(frameExportConfigWidget);
             dlg.setButtons(KoDialog::Ok | KoDialog::Cancel);
             if (dlg.exec() == QDialog::Accepted) {
-                KisConfig cfg(false);
                 m_wantsRenderWithHDR = frameExportConfigWidget->configuration()->getPropertyLazy("saveAsHDR", false);
-                cfg.setExportConfiguration("img_sequence/" + mimetype, frameExportConfigWidget->configuration());
+                saveLastUsedConfiguration("img_sequence/" + mimetype, frameExportConfigWidget->configuration());
             }
 
             frameExportConfigWidget->hide();
@@ -406,9 +463,6 @@ void KisDlgAnimationRenderer::sequenceMimeTypeOptionsClicked()
     }
 }
 
-inline int roundByTwo(int value) {
-    return value + (value & 0x1);
-}
 
 KisAnimationRenderingOptions KisDlgAnimationRenderer::getEncoderOptions() const
 {
@@ -417,6 +471,7 @@ KisAnimationRenderingOptions KisDlgAnimationRenderer::getEncoderOptions() const
     options.lastDocuemntPath = m_doc->localFilePath();
     options.videoMimeType = m_page->cmbRenderType->currentData().toString();
     options.frameMimeType = m_page->cmbMimetype->currentData().toString();
+    options.scaleFilter = m_page->cmbScaleFilter->currentData().toString();
 
     options.basename = m_page->txtBasename->text();
     options.directory = m_page->dirRequester->fileName();
@@ -436,16 +491,14 @@ KisAnimationRenderingOptions KisDlgAnimationRenderer::getEncoderOptions() const
         options.frameRate = 50;
     }
 
-    options.width = roundByTwo(m_page->intWidth->value());
-    options.height = roundByTwo(m_page->intHeight->value());
+    options.width = m_page->intWidth->value();
+    options.height = m_page->intHeight->value();
     options.videoFileName = m_page->videoFilename->fileName();
 
     options.customFFMpegOptions = m_customFFMpegOptionsString;
 
     {
-        KisConfig config(true);
-
-        KisPropertiesConfigurationSP cfg = config.exportConfiguration("img_sequence/" + options.frameMimeType);
+        KisPropertiesConfigurationSP cfg = loadLastConfiguration("img_sequence/" + options.frameMimeType);
         if (cfg) {
             KisImportExportManager::fillStaticExportConfigurationProperties(cfg, m_image);
         }
@@ -514,63 +567,11 @@ void KisDlgAnimationRenderer::slotDialogAccepted()
 {
     KisConfig cfg(false);
     KisAnimationRenderingOptions options = getEncoderOptions();
-    cfg.setExportConfiguration("ANIMATION_EXPORT", options.toProperties());
-}
+    saveLastUsedConfiguration("ANIMATION_EXPORT", options.toProperties());
 
-QString KisDlgAnimationRenderer::findFFMpeg(const QString &customLocation)
-{
-    QString result;
-
-    QStringList proposedPaths;
-
-    if (!customLocation.isEmpty()) {
-        proposedPaths << customLocation;
-        proposedPaths << customLocation + '/' + "ffmpeg";
-    }
-
-    proposedPaths << KoResourcePaths::getApplicationRoot() +
-        '/' + "bin" + '/' + "ffmpeg";
-
-#ifndef Q_OS_WIN
-    proposedPaths << QDir::homePath() + "/bin/ffmpeg";
-    proposedPaths << "/usr/bin/ffmpeg";
-    proposedPaths << "/usr/local/bin/ffmpeg";
-#endif
-
-    Q_FOREACH (QString path, proposedPaths) {
-        if (path.isEmpty()) continue;
-
-#ifdef Q_OS_WIN
-        path = QDir::toNativeSeparators(QDir::cleanPath(path));
-        if (path.endsWith('/')) {
-            continue;
-        }
-        if (!path.endsWith(".exe")) {
-            if (!QFile::exists(path)) {
-                path += ".exe";
-                if (!QFile::exists(path)) {
-                    continue;
-                }
-            }
-        }
-#endif
-        QProcess testProcess;
-        testProcess.start(path, QStringList() << "-version");
-        if (testProcess.waitForStarted(1000)) {
-            testProcess.waitForFinished(1000);
-        }
-
-        const bool successfulStart =
-            testProcess.state() == QProcess::NotRunning &&
-            testProcess.error() == QProcess::UnknownError;
-
-        if (successfulStart) {
-            result = path;
-            break;
-        }
-    }
-
-    return result;
+    m_image->animationInterface()->setExportSequenceBaseName(options.basename);
+    m_image->animationInterface()->setExportSequenceFilePath(options.directory);
+    m_image->animationInterface()->setExportInitialFrameNumber(options.sequenceStart);
 }
 
 void KisDlgAnimationRenderer::slotExportTypeChanged()
@@ -596,6 +597,8 @@ void KisDlgAnimationRenderer::slotExportTypeChanged()
     m_page->intHeight->setVisible(willEncodeVideo);
     m_page->intFramesPerSecond->setVisible(willEncodeVideo);
     m_page->fpsLabel->setVisible(willEncodeVideo);
+    m_page->cmbScaleFilter->setVisible(willEncodeVideo);
+    m_page->scaleFilterLabel->setVisible(willEncodeVideo);
     m_page->lblWidth->setVisible(willEncodeVideo);
     m_page->lblHeight->setVisible(willEncodeVideo);
 
@@ -631,7 +634,7 @@ void KisDlgAnimationRenderer::slotExportTypeChanged()
 void KisDlgAnimationRenderer::frameRateChanged(int framerate)
 {
     const QString mimeType = m_page->cmbRenderType->itemData(m_page->cmbRenderType->currentIndex()).toString();
-    m_page->lblGifWarning->setVisible((mimeType == "image/gif" && framerate > 50));
+    m_page->lblGifWarningFPS->setVisible((mimeType == "image/gif" && framerate > 50));
 }
 
 void KisDlgAnimationRenderer::slotLockAspectRatioDimensionsWidth(int width)
