@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2012 Dmitry Kazakov <dimula73@gmail.com>
+ *  SPDX-FileCopyrightText: 2012 Dmitry Kazakov <dimula73@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "kis_shortcut_matcher.h"
@@ -370,7 +358,7 @@ void KisShortcutMatcher::leaveEvent()
 
 bool KisShortcutMatcher::touchBeginEvent( QTouchEvent* event )
 {
-    Q_UNUSED(event)
+    Q_UNUSED(event);
 
     Private::RecursionNotifier notifier(this);
 
@@ -418,7 +406,7 @@ void KisShortcutMatcher::touchCancelEvent(const QPointF &localPos)
 
 bool KisShortcutMatcher::nativeGestureBeginEvent(QNativeGestureEvent *event)
 {
-    Q_UNUSED(event)
+    Q_UNUSED(event);
 
     Private::RecursionNotifier notifier(this);
 
@@ -525,6 +513,8 @@ void KisShortcutMatcher::lostFocusEvent(const QPointF &localPos)
     if (m_d->runningShortcut) {
         forceEndRunningShortcut(localPos);
     }
+
+    forceDeactivateAllActions();
 }
 
 void KisShortcutMatcher::reset()
