@@ -1,28 +1,16 @@
 /*
- *  Copyright (c) 2005-2007 Adrian Page <adrian@pagenet.plus.com>
+ *  SPDX-FileCopyrightText: 2005-2007 Adrian Page <adrian@pagenet.plus.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "opengl/kis_opengl_image_textures.h"
 
-#ifdef HAS_ONLY_OPENGL_ES
+#ifdef QT_OPENGL_ES_2
 #include <qopengl.h>
 #endif
 
-#ifndef HAS_ONLY_OPENGL_ES
+#ifndef QT_OPENGL_ES_2
 #include <QOpenGLFunctions>
 #endif
 #include <QOpenGLContext>
@@ -40,6 +28,10 @@
 #include "KisPart.h"
 #include "KisOpenGLModeProber.h"
 #include "kis_fixed_paint_device.h"
+#include "KisOpenGLSync.h"
+#include <QVector3D>
+#include "kis_painting_tweaks.h"
+#include "KisOpenGLBufferCreationGuard.h"
 
 #ifdef HAVE_OPENEXR
 #include <half.h>
@@ -61,6 +53,10 @@
 #define GL_BGRA8_EXT 0x93A1
 #endif
 
+#ifndef GL_RGBA16_EXT
+#define GL_RGBA16_EXT 0x805B
+#endif
+
 
 KisOpenGLImageTextures::ImageTexturesMap KisOpenGLImageTextures::imageTexturesMap;
 
@@ -68,6 +64,7 @@ KisOpenGLImageTextures::KisOpenGLImageTextures()
     : m_image(0)
     , m_monitorProfile(0)
     , m_internalColorManagementActive(true)
+    , m_bufferStorage(QOpenGLBuffer::PixelUnpackBuffer)
     , m_glFuncs(0)
     , m_useOcio(false)
     , m_initialized(false)
@@ -90,6 +87,7 @@ KisOpenGLImageTextures::KisOpenGLImageTextures(KisImageWSP image,
     , m_renderingIntent(renderingIntent)
     , m_conversionFlags(conversionFlags)
     , m_internalColorManagementActive(true)
+    , m_bufferStorage(QOpenGLBuffer::PixelUnpackBuffer)
     , m_glFuncs(0)
     , m_useOcio(false)
     , m_initialized(false)
@@ -156,6 +154,22 @@ bool KisOpenGLImageTextures::imageCanShareTextures()
     return true;
 }
 
+void KisOpenGLImageTextures::initBufferStorage(bool useBuffer)
+{
+    if (useBuffer) {
+        const int numTextureBuffers = 16;
+
+        const KoColorSpace *tilesDestinationColorSpace =
+            m_updateInfoBuilder.destinationColorSpace();
+        const int pixelSize = tilesDestinationColorSpace->pixelSize();
+        const int tileSize = m_texturesInfo.width * m_texturesInfo.height * pixelSize;
+
+        m_bufferStorage.allocate(numTextureBuffers, tileSize);
+    } else {
+        m_bufferStorage.reset();
+    }
+}
+
 KisOpenGLImageTexturesSP KisOpenGLImageTextures::getImageTextures(KisImageWSP image,
                                                                   const KoColorProfile *monitorProfile,
                                                                   KoColorConversionTransformation::Intent renderingIntent,
@@ -210,6 +224,8 @@ void KisOpenGLImageTextures::recreateImageTextureTiles()
     KisConfig config(true);
     KisOpenGL::FilterMode mode = (KisOpenGL::FilterMode)config.openGLFilteringMode();
 
+    initBufferStorage(config.useOpenGLTextureBuffer());
+
     QOpenGLContext *ctx = QOpenGLContext::currentContext();
     if (ctx) {
         QOpenGLFunctions *f = ctx->functions();
@@ -217,7 +233,14 @@ void KisOpenGLImageTextures::recreateImageTextureTiles()
         m_initialized = true;
         dbgUI  << "OpenGL: creating texture tiles of size" << m_texturesInfo.height << "x" << m_texturesInfo.width;
 
+        QVector<QRectF> tileImageRect;
+        QVector<QRectF> tileTextureRect;
+
         m_textureTiles.reserve((lastRow+1)*m_numCols);
+
+        tileImageRect.reserve(m_textureTiles.size());
+        tileTextureRect.reserve(m_textureTiles.size());
+
         for (int row = 0; row <= lastRow; row++) {
             for (int col = 0; col <= lastCol; col++) {
                 QRect tileRect = m_updateInfoBuilder.calculateEffectiveTileRect(col, row, m_image->bounds());
@@ -226,10 +249,40 @@ void KisOpenGLImageTextures::recreateImageTextureTiles()
                                                           &m_texturesInfo,
                                                           emptyTileData,
                                                           mode,
-                                                          config.useOpenGLTextureBuffer(),
+                                                          m_bufferStorage.isValid() ? &m_bufferStorage : 0,
                                                           config.numMipmapLevels(),
                                                           f);
                 m_textureTiles.append(tile);
+                tileImageRect.append(tile->tileRectInImagePixels());
+                tileTextureRect.append(tile->tileRectInTexturePixels());
+            }
+        }
+
+
+
+        {
+            KisOpenGLBufferCreationGuard bufferGuard(&m_tileVertexBuffer,
+                                                     tileImageRect.size() * 6 * 3 * sizeof(float),
+                                                     QOpenGLBuffer::StaticDraw);
+
+            QVector3D* mappedPtr = reinterpret_cast<QVector3D*>(bufferGuard.data());
+
+            Q_FOREACH (const QRectF &rc, tileImageRect) {
+                KisPaintingTweaks::rectToVertices(mappedPtr, rc);
+                mappedPtr += 6;
+            }
+        }
+
+        {
+            KisOpenGLBufferCreationGuard bufferGuard(&m_tileTexCoordBuffer,
+                                                     tileImageRect.size() * 6 * 2 * sizeof(float),
+                                                     QOpenGLBuffer::StaticDraw);
+
+            QVector2D* mappedPtr = reinterpret_cast<QVector2D*>(bufferGuard.data());
+
+            Q_FOREACH (const QRectF &rc, tileTextureRect) {
+                KisPaintingTweaks::rectToTexCoords(mappedPtr, rc);
+                mappedPtr += 6;
             }
         }
     }
@@ -246,6 +299,8 @@ void KisOpenGLImageTextures::destroyImageTextureTiles()
         delete tile;
     }
     m_textureTiles.clear();
+    m_tileVertexBuffer.destroy();
+    m_tileTexCoordBuffer.destroy();
     m_storedImageBounds = QRect();
 }
 
@@ -276,12 +331,36 @@ void KisOpenGLImageTextures::recalculateCache(KisUpdateInfoSP info, bool blockMi
     KisOpenGLUpdateInfoSP glInfo = dynamic_cast<KisOpenGLUpdateInfo*>(info.data());
     if(!glInfo) return;
 
+    QScopedPointer<KisOpenGLSync> sync;
+    int numProcessedTiles = 0;
+
     KisTextureTileUpdateInfoSP tileInfo;
     Q_FOREACH (tileInfo, glInfo->tileList) {
         KisTextureTile *tile = getTextureTileCR(tileInfo->tileCol(), tileInfo->tileRow());
         KIS_ASSERT_RECOVER_RETURN(tile);
 
+        if (m_bufferStorage.isValid() && numProcessedTiles > m_bufferStorage.size() &&
+            sync && !sync->isSignaled()) {
+
+            qDebug() << "Still unsignalled after processed" << numProcessedTiles << "tiles";
+
+            const int nextSize = qNextPowerOfTwo(m_bufferStorage.size());
+            m_bufferStorage.allocateMoreBuffers(nextSize);
+            qDebug() << "    increased number of buffers to" << nextSize;
+        }
+
+
         tile->update(*tileInfo, blockMipmapRegeneration);
+
+        if (m_bufferStorage.isValid()) {
+            if (!sync) {
+                sync.reset(new KisOpenGLSync());
+                numProcessedTiles = 0;
+            } else if (sync && sync->isSignaled()) {
+                sync.reset();
+            }
+            numProcessedTiles++;
+        }
     }
 }
 
@@ -355,8 +434,10 @@ void KisOpenGLImageTextures::updateConfig(bool useBuffer, int NumMipmapLevels)
 {
     if(m_textureTiles.isEmpty()) return;
 
+    initBufferStorage(useBuffer);
+
     Q_FOREACH (KisTextureTile *tile, m_textureTiles) {
-        tile->setUseBuffer(useBuffer);
+        tile->setBufferStorage(useBuffer ? &m_bufferStorage : 0);
         tile->setNumMipmapLevels(NumMipmapLevels);
     }
 }
@@ -497,49 +578,81 @@ bool KisOpenGLImageTextures::setInternalColorManagementActive(bool value)
 namespace {
 void initializeRGBA16FTextures(QOpenGLContext *ctx, KisGLTexturesInfo &texturesInfo, KoID &destinationColorDepthId)
 {
-    if (KisOpenGL::hasOpenGLES() || KisOpenGL::hasOpenGL3()) {
-        texturesInfo.internalFormat = GL_RGBA16F;
-        dbgUI << "Using half (GLES or GL3)";
-    } else if (ctx->hasExtension("GL_ARB_texture_float")) {
-#ifndef HAS_ONLY_OPENGL_ES
-        texturesInfo.internalFormat = GL_RGBA16F_ARB;
-        dbgUI << "Using ARB half";
-    }
-    else if (ctx->hasExtension("GL_ATI_texture_float")) {
-        texturesInfo.internalFormat = GL_RGBA_FLOAT16_ATI;
-        dbgUI << "Using ATI half";
-#else
-        KIS_ASSERT_X(false, "initializeRGBA16FTextures",
-                "Unexpected KisOpenGL::hasOpenGLES and \
-                 KisOpenGL::hasOpenGL3 returned false");
-#endif
-    }
-
     bool haveBuiltInOpenExr = false;
 #ifdef HAVE_OPENEXR
     haveBuiltInOpenExr = true;
 #endif
 
     if (haveBuiltInOpenExr && (KisOpenGL::hasOpenGLES() || KisOpenGL::hasOpenGL3())) {
+#ifndef QT_OPENGL_ES_2
+        texturesInfo.internalFormat = GL_RGBA16F;
+        dbgUI << "Using half (GLES or GL3)";
         texturesInfo.type = GL_HALF_FLOAT;
         destinationColorDepthId = Float16BitsColorDepthID;
         dbgUI << "Pixel type half (GLES or GL3)";
+        texturesInfo.format = GL_RGBA;
     } else if (haveBuiltInOpenExr && ctx->hasExtension("GL_ARB_half_float_pixel")) {
-#ifndef HAS_ONLY_OPENGL_ES
+        texturesInfo.internalFormat = GL_RGBA16F_ARB;
+        dbgUI << "Using ARB half";
         texturesInfo.type = GL_HALF_FLOAT_ARB;
         destinationColorDepthId = Float16BitsColorDepthID;
+        texturesInfo.format = GL_RGBA;
         dbgUI << "Pixel type half";
+    } else if (haveBuiltInOpenExr && ctx->hasExtension("GL_ATI_texture_float")) {
+        texturesInfo.internalFormat = GL_RGBA_FLOAT16_ATI;
+        dbgUI << "Using ATI half";
+        texturesInfo.type = GL_HALF_FLOAT;
+        destinationColorDepthId = Float16BitsColorDepthID;
+        dbgUI << "Using half (GLES or GL3)";
+        texturesInfo.format = GL_RGBA;
     } else {
+        texturesInfo.internalFormat = GL_RGBA32F;
         texturesInfo.type = GL_FLOAT;
         destinationColorDepthId = Float32BitsColorDepthID;
         dbgUI << "Pixel type float";
+        texturesInfo.format = GL_RGBA;
 #else
-        KIS_ASSERT_X(false, "KisOpenGLCanvas2::paintToolOutline",
-                "Unexpected KisOpenGL::hasOpenGLES and \
-                 KisOpenGL::hasOpenGL3 returned false");
+        if (ctx->format().majorVersion() >= 3) {
+            texturesInfo.internalFormat = GL_RGBA16F;
+            dbgUI << "Using half (GLES 3 non-Angle)";
+            texturesInfo.type = GL_HALF_FLOAT;
+            destinationColorDepthId = Float16BitsColorDepthID;
+            dbgUI << "Pixel type half (GLES 3 non-Angle)";
+            texturesInfo.format = GL_RGBA;
+        } else if (ctx->hasExtension("GL_OES_texture_half_float") && ctx->hasExtension("GL_EXT_color_buffer_half_float")
+            && ctx->hasExtension("GL_OES_texture_half_float_linear")) {
+            texturesInfo.internalFormat = GL_RGBA16F_EXT;
+            dbgUI << "Using half (GLES v2)";
+            texturesInfo.type = GL_HALF_FLOAT_OES;
+            destinationColorDepthId = Float16BitsColorDepthID;
+            dbgUI << "Pixel type half (GLES v2)";
+            texturesInfo.format = GL_RGBA;
+        } else if (ctx->hasExtension("GL_OES_texture_float") && (ctx->hasExtension("GL_EXT_texture_storage") || ctx->hasExtension("EXT_color_buffer_float"))
+                   && ctx->hasExtension("GL_OES_texture_float_linear")) {
+            texturesInfo.internalFormat = GL_RGBA32F_EXT;
+            dbgUI << "Using float (GLES v2)";
+            texturesInfo.type = GL_FLOAT;
+            destinationColorDepthId = Float32BitsColorDepthID;
+            dbgUI << "Pixel type float (GLES v2)";
+            texturesInfo.format = GL_RGBA;
+        }
+    } else if (ctx->format().majorVersion() >= 3) {
+        texturesInfo.internalFormat = GL_RGBA32F;
+        dbgUI << "Using float (GLES 3 non-Angle)";
+        texturesInfo.type = GL_FLOAT;
+        destinationColorDepthId = Float32BitsColorDepthID;
+        dbgUI << "Pixel type float (GLES 3 non-Angle)";
+        texturesInfo.format = GL_RGBA;
+    } else if (ctx->hasExtension("GL_OES_texture_float") && (ctx->hasExtension("GL_EXT_texture_storage") || ctx->hasExtension("EXT_color_buffer_float"))
+            && ctx->hasExtension("GL_OES_texture_float_linear")) {
+        texturesInfo.internalFormat = GL_RGBA32F_EXT;
+        dbgUI << "Using float (GLES v2)";
+        texturesInfo.type = GL_FLOAT;
+        destinationColorDepthId = Float32BitsColorDepthID;
+        dbgUI << "Pixel type float (GLES v2)";
+        texturesInfo.format = GL_RGBA;
 #endif
     }
-    texturesInfo.format = GL_RGBA;
 }
 }
 
@@ -549,7 +662,7 @@ void KisOpenGLImageTextures::updateTextureFormat()
     if (!(m_image && ctx)) return;
 
     if (!KisOpenGL::hasOpenGLES()) {
-#ifndef HAS_ONLY_OPENGL_ES
+#ifndef QT_OPENGL_ES_2
         m_texturesInfo.internalFormat = GL_RGBA8;
         m_texturesInfo.type = GL_UNSIGNED_BYTE;
         m_texturesInfo.format = GL_BGRA;
@@ -558,7 +671,7 @@ void KisOpenGLImageTextures::updateTextureFormat()
                 "Unexpected KisOpenGL::hasOpenGLES returned false");
 #endif
     } else {
-#ifdef HAS_ONLY_OPENGL_ES
+#ifdef QT_OPENGL_ES_2
         m_texturesInfo.internalFormat = GL_RGBA8;
         m_texturesInfo.type = GL_UNSIGNED_BYTE;
         m_texturesInfo.format = GL_RGBA;
@@ -591,57 +704,107 @@ void KisOpenGLImageTextures::updateTextureFormat()
         }
         else if (colorDepthId == Float32BitsColorDepthID) {
             if (KisOpenGL::hasOpenGLES() || KisOpenGL::hasOpenGL3()) {
+#ifndef QT_OPENGL_ES_2
                 m_texturesInfo.internalFormat = GL_RGBA32F;
                 dbgUI << "Using float (GLES or GL3)";
+                m_texturesInfo.type = GL_FLOAT;
+                m_texturesInfo.format = GL_RGBA;
+                destinationColorDepthId = Float32BitsColorDepthID;
             } else if (ctx->hasExtension("GL_ARB_texture_float")) {
-#ifndef HAS_ONLY_OPENGL_ES
                 m_texturesInfo.internalFormat = GL_RGBA32F_ARB;
                 dbgUI << "Using ARB float";
+                m_texturesInfo.type = GL_FLOAT;
+                m_texturesInfo.format = GL_RGBA;
+                destinationColorDepthId = Float32BitsColorDepthID;
             } else if (ctx->hasExtension("GL_ATI_texture_float")) {
                 m_texturesInfo.internalFormat = GL_RGBA_FLOAT32_ATI;
                 dbgUI << "Using ATI float";
+                m_texturesInfo.type = GL_FLOAT;
+                m_texturesInfo.format = GL_RGBA;
+                destinationColorDepthId = Float32BitsColorDepthID;
 #else
-        KIS_ASSERT_X(false, "KisOpenGLCanvas2::updateTextureFormat",
-                "Unexpected KisOpenGL::hasOpenGLES and \
-                 KisOpenGL::hasOpenGL3 returned false");
+                if (ctx->format().majorVersion() >= 3) {
+                    m_texturesInfo.internalFormat = GL_RGBA32F;
+                    dbgUI << "Using float (GLES 3 non-Angle)";
+                    m_texturesInfo.type = GL_FLOAT;
+                    m_texturesInfo.format = GL_RGBA;
+                    destinationColorDepthId = Float32BitsColorDepthID;
+                    /* GL_EXT_texture_storage is GLES 2*/
+                    /* GL_EXT_color_buffer_float is GLES 3*/
+                } else if (ctx->hasExtension("GL_OES_texture_float")
+                           && (ctx->hasExtension("GL_EXT_texture_storage") || ctx->hasExtension("EXT_color_buffer_float"))
+                           && ctx->hasExtension("GL_OES_texture_float_linear")) {
+                    m_texturesInfo.internalFormat = GL_RGBA32F_EXT;
+                    dbgUI << "Using float (GLES v2)";
+                    m_texturesInfo.type = GL_FLOAT;
+                    m_texturesInfo.format = GL_RGBA;
+                    destinationColorDepthId = Float32BitsColorDepthID;
+                }
 #endif
             }
-
-            m_texturesInfo.type = GL_FLOAT;
-            m_texturesInfo.format = GL_RGBA;
-            destinationColorDepthId = Float32BitsColorDepthID;
         }
         else if (colorDepthId == Integer16BitsColorDepthID) {
-#ifndef HAS_ONLY_OPENGL_ES
             if (!KisOpenGL::hasOpenGLES()) {
+#ifndef QT_OPENGL_ES_2
                 m_texturesInfo.internalFormat = GL_RGBA16;
                 m_texturesInfo.type = GL_UNSIGNED_SHORT;
                 m_texturesInfo.format = GL_BGRA;
                 destinationColorDepthId = Integer16BitsColorDepthID;
                 dbgUI << "Using 16 bits rgba";
-            }
+#else
+                KIS_ASSERT_X(false,
+                             "KisOpenGLCanvas2::updateTextureFormat",
+                             "Unexpected KisOpenGL::hasOpenGLES returned false");
 #endif
-            // TODO: for ANGLE, see if we can convert to 16f to support 10-bit display
+            } else {
+#ifdef QT_OPENGL_ES_2_ANGLE
+                // If OpenGL ES, fall back to 16-bit float -- supports HDR
+                // Angle does ship GL_EXT_texture_norm16 but it doesn't seem
+                // to be renderable by DXGI - it returns a pixel size of 0
+                initializeRGBA16FTextures(ctx, m_texturesInfo, destinationColorDepthId);
+#else
+                if (ctx->hasExtension("GL_EXT_texture_norm16")) {
+                    m_texturesInfo.internalFormat = GL_RGBA16_EXT;
+                    m_texturesInfo.type = GL_UNSIGNED_SHORT;
+                    destinationColorDepthId = Integer16BitsColorDepthID;
+                    dbgUI << "Using 16 bits rgba (GLES v2)";
+                }
+#endif
+            }
         }
     }
     else {
         // We will convert the colorspace to 16 bits rgba, instead of 8 bits
-        if (colorDepthId == Integer16BitsColorDepthID && !KisOpenGL::hasOpenGLES()) {
-#ifndef HAS_ONLY_OPENGL_ES
-            m_texturesInfo.internalFormat = GL_RGBA16;
-            m_texturesInfo.type = GL_UNSIGNED_SHORT;
-            m_texturesInfo.format = GL_BGRA;
-            destinationColorDepthId = Integer16BitsColorDepthID;
-            dbgUI << "Using conversion to 16 bits rgba";
+        if (colorDepthId == Integer16BitsColorDepthID) {
+            if (!KisOpenGL::hasOpenGLES()) {
+#ifndef QT_OPENGL_ES_2
+                m_texturesInfo.internalFormat = GL_RGBA16;
+                m_texturesInfo.type = GL_UNSIGNED_SHORT;
+                m_texturesInfo.format = GL_BGRA;
+                destinationColorDepthId = Integer16BitsColorDepthID;
+                dbgUI << "Using conversion to 16 bits rgba";
 #else
-            KIS_ASSERT_X(false, "KisOpenGLCanvas2::updateTextureFormat",
+                KIS_ASSERT_X(false, "KisOpenGLCanvas2::updateTextureFormat",
                     "Unexpected KisOpenGL::hasOpenGLES returned false");
 #endif
-        } else if (colorDepthId == Float16BitsColorDepthID && KisOpenGL::hasOpenGLES()) {
-            // TODO: try removing opengl es limit
+            } else {
+#ifdef QT_OPENGL_ES_2_ANGLE
+                // If OpenGL ES, fall back to 16-bit float -- supports HDR
+                // Angle does ship GL_EXT_texture_norm16 but it doesn't seem
+                // to be renderable by DXGI - it returns a pixel size of 0
+                initializeRGBA16FTextures(ctx, m_texturesInfo, destinationColorDepthId);
+#else
+                if (ctx->hasExtension("GL_EXT_texture_norm16")) {
+                    m_texturesInfo.internalFormat = GL_RGBA16_EXT;
+                    m_texturesInfo.type = GL_UNSIGNED_SHORT;
+                    destinationColorDepthId = Integer16BitsColorDepthID;
+                    dbgUI << "Using conversion to 16 bits rgba (GLES v2)";
+                }
+#endif
+            }
+        } else if (colorDepthId == Float16BitsColorDepthID) {
             initializeRGBA16FTextures(ctx, m_texturesInfo, destinationColorDepthId);
         }
-        // TODO: for ANGLE, see if we can convert to 16f to support 10-bit display
     }
 
     if (!m_internalColorManagementActive &&
