@@ -1,24 +1,13 @@
 /*
- *  Copyright (c) 2010 Dmitry Kazakov <dimula73@gmail.com>
+ *  SPDX-FileCopyrightText: 2010 Dmitry Kazakov <dimula73@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #define GL_GLEXT_PROTOTYPES
 #include "kis_texture_tile.h"
 #include "kis_texture_tile_update_info.h"
+#include "KisOpenGLBufferCircularStorage.h"
 
 #include <kis_debug.h>
 #if !defined(QT_OPENGL_ES)
@@ -29,6 +18,7 @@
 #define GL_BGRA 0x814F
 #endif
 
+
 void KisTextureTile::setTextureParameters()
 {
 
@@ -39,7 +29,11 @@ void KisTextureTile::setTextureParameters()
     f->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
     f->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, m_numMipmapLevels);
 
-    if (m_texturesInfo->internalFormat == GL_RGBA8 && m_texturesInfo->format == GL_RGBA) {
+    if ((m_texturesInfo->internalFormat == GL_RGBA8 && m_texturesInfo->format == GL_RGBA)
+#ifdef QT_OPENGL_ES_3
+        || (m_texturesInfo->internalFormat == GL_RGBA16_EXT && m_texturesInfo->format == GL_RGBA)
+#endif
+    ) {
         // If image format is RGBA8, swap the red and blue channels for the proper colour
         // This is for OpenGL ES support and only used if lacking GL_EXT_texture_format_BGRA8888
         f->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_R, GL_BLUE);
@@ -61,23 +55,21 @@ inline QRectF relativeRect(const QRect &br /* baseRect */,
     return QRectF(x, y, w, h);
 }
 
+#include "kis_debug.h"
 
 KisTextureTile::KisTextureTile(const QRect &imageRect, const KisGLTexturesInfo *texturesInfo,
                                const QByteArray &fillData, KisOpenGL::FilterMode filter,
-                               bool useBuffer, int numMipmapLevels, QOpenGLFunctions *fcn)
+                               KisOpenGLBufferCircularStorage *bufferStorage, int numMipmapLevels, QOpenGLFunctions *fcn)
 
     : m_textureId(0)
-#ifdef USE_PIXEL_BUFFERS
-    , m_glBuffer(0)
-#endif
     , m_tileRectInImagePixels(imageRect)
     , m_filter(filter)
     , m_texturesInfo(texturesInfo)
     , m_needsMipmapRegeneration(false)
     , m_preparedLodPlane(0)
-    , m_useBuffer(useBuffer)
     , m_numMipmapLevels(numMipmapLevels)
     , f(fcn)
+    , m_bufferStorage(bufferStorage)
 {
     const GLvoid *fd = fillData.constData();
 
@@ -93,11 +85,8 @@ KisTextureTile::KisTextureTile(const QRect &imageRect, const KisGLTexturesInfo *
 
     setTextureParameters();
 
-#ifdef USE_PIXEL_BUFFERS
-    createTextureBuffer(fillData.constData(), fillData.size());
-    // we set fill data to 0 so the next glTexImage2D call uses our buffer
-    fd = 0;
-#endif
+    KisOpenGLBufferCircularStorage::BufferBinder binder(
+        m_bufferStorage, &fd, fillData.size());
 
     f->glTexImage2D(GL_TEXTURE_2D, 0,
                  m_texturesInfo->internalFormat,
@@ -106,20 +95,11 @@ KisTextureTile::KisTextureTile(const QRect &imageRect, const KisGLTexturesInfo *
                  m_texturesInfo->format,
                  m_texturesInfo->type, fd);
 
-#ifdef USE_PIXEL_BUFFERS
-    if (m_useBuffer && m_glBuffer) {
-        m_glBuffer->release();
-    }
-#endif
-
     setNeedsMipmapRegeneration();
 }
 
 KisTextureTile::~KisTextureTile()
 {
-#ifdef USE_PIXEL_BUFFERS
-    delete m_glBuffer;
-#endif
     f->glDeleteTextures(1, &m_textureId);
 }
 
@@ -162,11 +142,6 @@ void KisTextureTile::update(const KisTextureTileUpdateInfo &updateInfo, bool blo
     const QPoint patchOffset = updateInfo.realPatchOffset();
 
     const GLvoid *fd = updateInfo.data();
-#ifdef USE_PIXEL_BUFFERS
-    if (!m_glBuffer) {
-        createTextureBuffer((const char*)updateInfo.data(), updateInfo.patchPixelsLength());
-    }
-#endif
 
     /**
      * In some special case, when the Lod0 stroke is cancelled the
@@ -199,21 +174,8 @@ void KisTextureTile::update(const KisTextureTileUpdateInfo &updateInfo, bool blo
 
 
     if (updateInfo.isEntireTileUpdated()) {
-
-#ifdef USE_PIXEL_BUFFERS
-        if (m_useBuffer) {
-
-            m_glBuffer->bind();
-            m_glBuffer->allocate(updateInfo.patchPixelsLength());
-
-            void *vid = m_glBuffer->map(QOpenGLBuffer::WriteOnly);
-            memcpy(vid, fd, updateInfo.patchPixelsLength());
-            m_glBuffer->unmap();
-
-            // we set fill data to 0 so the next glTexImage2D call uses our buffer
-            fd = 0;
-        }
-#endif
+        KisOpenGLBufferCircularStorage::BufferBinder b(
+            m_bufferStorage, &fd, updateInfo.patchPixelsLength());
 
         f->glTexImage2D(GL_TEXTURE_2D, patchLevelOfDetail,
                      m_texturesInfo->internalFormat,
@@ -222,29 +184,11 @@ void KisTextureTile::update(const KisTextureTileUpdateInfo &updateInfo, bool blo
                      m_texturesInfo->format,
                      m_texturesInfo->type,
                      fd);
-
-#ifdef USE_PIXEL_BUFFERS
-        if (m_useBuffer) {
-            m_glBuffer->release();
-        }
-#endif
-
     }
     else {
-#ifdef USE_PIXEL_BUFFERS
-        if (m_useBuffer) {
-            m_glBuffer->bind();
-            int size = patchSize.width() * patchSize.height() * updateInfo.pixelSize();
-            m_glBuffer->allocate(size);
-
-            void *vid = m_glBuffer->map(QOpenGLBuffer::WriteOnly);
-            memcpy(vid, fd, size);
-            m_glBuffer->unmap();
-
-            // we set fill data to 0 so the next glTexImage2D call uses our buffer
-            fd = 0;
-        }
-#endif
+        const int size = patchSize.width() * patchSize.height() * updateInfo.pixelSize();
+        KisOpenGLBufferCircularStorage::BufferBinder b(
+            m_bufferStorage, &fd, size);
 
         f->glTexSubImage2D(GL_TEXTURE_2D, patchLevelOfDetail,
                         patchOffset.x(), patchOffset.y(),
@@ -253,12 +197,6 @@ void KisTextureTile::update(const KisTextureTileUpdateInfo &updateInfo, bool blo
                         m_texturesInfo->type,
                         fd);
 
-#ifdef USE_PIXEL_BUFFERS
-        if (m_useBuffer) {
-            m_glBuffer->release();
-        }
-#endif
-
     }
 
     /**
@@ -278,13 +216,19 @@ void KisTextureTile::update(const KisTextureTileUpdateInfo &updateInfo, bool blo
     if(updateInfo.isTopmost()) {
         int start = 0;
         int end = patchOffset.y() - 1;
+
+        const GLvoid *fd = updateInfo.data();
+        const int size = patchSize.width() * pixelSize;
+        KisOpenGLBufferCircularStorage::BufferBinder g(
+            m_bufferStorage, &fd, size);
+
         for (int i = start; i <= end; i++) {
             f->glTexSubImage2D(GL_TEXTURE_2D, patchLevelOfDetail,
-                            patchOffset.x(), i,
-                            patchSize.width(), 1,
-                            m_texturesInfo->format,
-                            m_texturesInfo->type,
-                            updateInfo.data());
+                               patchOffset.x(), i,
+                               patchSize.width(), 1,
+                               m_texturesInfo->format,
+                               m_texturesInfo->type,
+                               fd);
         }
     }
 
@@ -294,13 +238,19 @@ void KisTextureTile::update(const KisTextureTileUpdateInfo &updateInfo, bool blo
 
         int start = patchOffset.y() + patchSize.height();
         int end = tileSize.height() - 1;
+
+        const GLvoid *fd = updateInfo.data() + shift;
+        const int size = patchSize.width() * pixelSize;
+        KisOpenGLBufferCircularStorage::BufferBinder g(
+            m_bufferStorage, &fd, size);
+
         for (int i = start; i < end; i++) {
             f->glTexSubImage2D(GL_TEXTURE_2D, patchLevelOfDetail,
                             patchOffset.x(), i,
                             patchSize.width(), 1,
                             m_texturesInfo->format,
                             m_texturesInfo->type,
-                            updateInfo.data() + shift);
+                            fd);
         }
     }
 
@@ -319,13 +269,19 @@ void KisTextureTile::update(const KisTextureTileUpdateInfo &updateInfo, bool blo
 
         int start = 0;
         int end = patchOffset.x() - 1;
+
+        const GLvoid *fd = columnBuffer.constData();
+        const int size = columnBuffer.size();
+        KisOpenGLBufferCircularStorage::BufferBinder g(
+            m_bufferStorage, &fd, size);
+
         for (int i = start; i <= end; i++) {
             f->glTexSubImage2D(GL_TEXTURE_2D, patchLevelOfDetail,
                             i, patchOffset.y(),
                             1, patchSize.height(),
                             m_texturesInfo->format,
                             m_texturesInfo->type,
-                            columnBuffer.constData());
+                            fd);
         }
     }
 
@@ -344,13 +300,19 @@ void KisTextureTile::update(const KisTextureTileUpdateInfo &updateInfo, bool blo
 
         int start = patchOffset.x() + patchSize.width();
         int end = tileSize.width() - 1;
+
+        const GLvoid *fd = columnBuffer.constData();
+        const int size = columnBuffer.size();
+        KisOpenGLBufferCircularStorage::BufferBinder g(
+            m_bufferStorage, &fd, size);
+
         for (int i = start; i <= end; i++) {
             f->glTexSubImage2D(GL_TEXTURE_2D, patchLevelOfDetail,
                             i, patchOffset.y(),
                             1, patchSize.height(),
                             m_texturesInfo->format,
                             m_texturesInfo->type,
-                            columnBuffer.constData());
+                            fd);
         }
     }
 
@@ -377,26 +339,3 @@ QRectF KisTextureTile::imageRectInTexturePixels(const QRect &imageRect) const
                         m_texturesInfo);
 
 }
-
-#ifdef USE_PIXEL_BUFFERS
-void KisTextureTile::createTextureBuffer(const char *data, int size)
-{
-    if (m_useBuffer) {
-        if (!m_glBuffer) {
-            m_glBuffer = new QOpenGLBuffer(QOpenGLBuffer::PixelUnpackBuffer);
-            m_glBuffer->setUsagePattern(QOpenGLBuffer::DynamicDraw);
-            m_glBuffer->create();
-            m_glBuffer->bind();
-            m_glBuffer->allocate(size);
-        }
-        void *vid = m_glBuffer->map(QOpenGLBuffer::WriteOnly);
-        memcpy(vid, data, size);
-        m_glBuffer->unmap();
-
-    }
-    else {
-        delete m_glBuffer;
-        m_glBuffer = 0;
-    }
-}
-#endif
