@@ -1,22 +1,10 @@
 /*
- * Copyright (c) 2008 Cyrille Berger <cberger@cberger.net>
- * Copyright (c) 2010 Geoffry Song <goffrie@gmail.com>
- * Copyright (c) 2014 Wolthera van Hövell tot Westerflier <griffinvalley@gmail.com>
- * Copyright (c) 2017 Scott Petrovic <scottpetrovic@gmail.com>
+ * SPDX-FileCopyrightText: 2008 Cyrille Berger <cberger@cberger.net>
+ * SPDX-FileCopyrightText: 2010 Geoffry Song <goffrie@gmail.com>
+ * SPDX-FileCopyrightText: 2014 Wolthera van Hövell tot Westerflier <griffinvalley@gmail.com>
+ * SPDX-FileCopyrightText: 2017 Scott Petrovic <scottpetrovic@gmail.com>
  *
- *  This library is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License as published by
- *  the Free Software Foundation; version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #include "InfiniteRulerAssistant.h"
@@ -78,7 +66,7 @@ QPointF InfiniteRulerAssistant::project(const QPointF& pt, const QPointF& stroke
     //return pt;
 }
 
-QPointF InfiniteRulerAssistant::adjustPosition(const QPointF& pt, const QPointF& strokeBegin)
+QPointF InfiniteRulerAssistant::adjustPosition(const QPointF& pt, const QPointF& strokeBegin, const bool /*snapToAny*/)
 {
     return project(pt, strokeBegin);
 }
@@ -106,7 +94,7 @@ void InfiniteRulerAssistant::drawAssistant(QPainter& gc, const QRectF& updateRec
         QTransform initialTransform = converter->documentToWidgetTransform();
         QLineF snapLine= QLineF(initialTransform.map(*handles()[0]), initialTransform.map(*handles()[1]));
         QRect viewport= gc.viewport();
-        KisAlgebra2D::intersectLineRect(snapLine, viewport);
+        KisAlgebra2D::intersectLineRect(snapLine, viewport, true);
         QPainterPath path;
         path.moveTo(snapLine.p1());
         path.lineTo(snapLine.p2());
