@@ -1,18 +1,7 @@
 /*
- *   Copyright (C) 2011 by Siddharth Sharma <siddharth.kde@gmail.com>
+ *   SPDX-FileCopyrightText: 2011 Siddharth Sharma <siddharth.kde@gmail.com>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program.  If not, see <http://www.gnu.org/licenses/>
+ *   SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include <psd_image_data.h>
@@ -57,7 +46,7 @@ bool PSDImageData::read(QIODevice *io, KisPaintDeviceSP dev ) {
     psdread(io, &m_compression);
     quint64 start = io->pos();
     m_channelSize = m_header->channelDepth/8;
-    m_channelDataLength = m_header->height * m_header->width * m_channelSize;
+    m_channelDataLength = quint64(m_header->height) * m_header->width * m_channelSize;
 
     dbgFile << "Reading Image Data Block: compression" << m_compression << "channelsize" << m_channelSize << "number of channels" << m_header->nChannels;
 
@@ -71,7 +60,7 @@ bool PSDImageData::read(QIODevice *io, KisPaintDeviceSP dev ) {
             channelInfo.channelId = channel;
             channelInfo.compressionType = Compression::Uncompressed;
             channelInfo.channelDataStart = start;
-            channelInfo.channelDataLength = m_header->width * m_header->height * m_channelSize;
+            channelInfo.channelDataLength = quint64(m_header->width) * m_header->height * m_channelSize;
             start += channelInfo.channelDataLength;
             m_channelInfoRecords.append(channelInfo);
 
@@ -93,14 +82,16 @@ bool PSDImageData::read(QIODevice *io, KisPaintDeviceSP dev ) {
 
         for (int channel = 0; channel < m_header->nChannels; channel++) {
             m_channelOffsets << 0;
-            quint32 sumrlelength = 0;
+            quint64 sumrlelength = 0;
             ChannelInfo channelInfo;
             channelInfo.channelId = channel;
             channelInfo.channelDataStart = start;
             channelInfo.compressionType = Compression::RLE;
             for (quint32 row = 0; row < m_header->height; row++ ) {
                 if (m_header->version == 1) {
-                    psdread(io,(quint16*)&rlelength);
+                    quint16 rlelength16; // use temporary variable to not cast pointers and not rely on endianness
+                    psdread(io,&rlelength16);
+                    rlelength = rlelength16;
                 }
                 else if (m_header->version == 2) {
                     psdread(io,&rlelength);
