@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2008 Boudewijn Rempt <boud@valdyas.org>
+ *  SPDX-FileCopyrightText: 2008 Boudewijn Rempt <boud@valdyas.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 #include "kis_duplicateop_option.h"
 #include <klocalizedstring.h>
@@ -52,6 +40,7 @@ KisDuplicateOpOption::KisDuplicateOpOption()
     connect(m_optionWidget->cbHealing, SIGNAL(toggled(bool)), SLOT(emitSettingChanged()));
     connect(m_optionWidget->cbPerspective, SIGNAL(toggled(bool)), SLOT(emitSettingChanged()));
     connect(m_optionWidget->cbSourcePoint, SIGNAL(toggled(bool)), SLOT(emitSettingChanged()));
+    connect(m_optionWidget->cbResetSourcePoint, SIGNAL(toggled(bool)), SLOT(emitSettingChanged()));
     connect(m_optionWidget->chkCloneProjection, SIGNAL(toggled(bool)), SLOT(emitSettingChanged()));
 
     setConfigurationPage(m_optionWidget);
@@ -92,6 +81,16 @@ void KisDuplicateOpOption::setMoveSourcePoint(bool move)
     m_optionWidget->cbSourcePoint->setChecked(move);
 }
 
+bool KisDuplicateOpOption::resetSourcePoint() const
+{
+    return m_optionWidget->cbResetSourcePoint->isChecked();
+}
+
+void KisDuplicateOpOption::setResetSourcePoint(bool reset)
+{
+    m_optionWidget->cbResetSourcePoint->setChecked(reset);
+}
+
 bool KisDuplicateOpOption::cloneFromProjection() const
 {
     return m_optionWidget->chkCloneProjection->isChecked();
@@ -109,6 +108,7 @@ void KisDuplicateOpOption::writeOptionSetting(KisPropertiesConfigurationSP setti
     op.duplicate_healing = healing();
     op.duplicate_correct_perspective = correctPerspective();
     op.duplicate_move_source_point = moveSourcePoint();
+    op.duplicate_reset_source_point = resetSourcePoint();
     op.duplicate_clone_from_projection = cloneFromProjection();
 
     op.writeOptionSetting(setting);
@@ -122,6 +122,7 @@ void KisDuplicateOpOption::readOptionSetting(const KisPropertiesConfigurationSP
     m_optionWidget->cbHealing->setChecked(op.duplicate_healing);
     m_optionWidget->cbPerspective->setChecked(op.duplicate_correct_perspective);
     m_optionWidget->cbSourcePoint->setChecked(op.duplicate_move_source_point);
+    m_optionWidget->cbResetSourcePoint->setChecked(op.duplicate_reset_source_point);
     m_optionWidget->chkCloneProjection->setChecked(op.duplicate_clone_from_projection);
 }
 
