@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2009,2010 Lukáš Tvrdý <lukast.dev@gmail.com>
+ *  SPDX-FileCopyrightText: 2009, 2010 Lukáš Tvrdý <lukast.dev@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 #include "kis_experiment_paintop_settings_widget.h"
 
@@ -43,8 +31,7 @@ KisExperimentPaintOpSettingsWidget::~ KisExperimentPaintOpSettingsWidget()
 
 KisPropertiesConfigurationSP  KisExperimentPaintOpSettingsWidget::configuration() const
 {
-    KisExperimentPaintOpSettings* config = new KisExperimentPaintOpSettings();
-    config->setOptionsWidget(const_cast<KisExperimentPaintOpSettingsWidget*>(this));
+    KisExperimentPaintOpSettings* config = new KisExperimentPaintOpSettings(resourcesInterface());
     config->setProperty("paintop", "experimentbrush"); // XXX: make this a const id string
     writeConfiguration(config);
     return config;
