@@ -1,22 +1,9 @@
 /* This file is part of the KDE project
- * Copyright (C) 2008 Boudewijn Rempt <boud@valdyas.org>
- * Copyright (C) 2009 Sven Langkamp   <sven.langkamp@gmail.com>
- * Copyright (C) 2011 Silvio Heinrich <plassy@web.de>
+ * SPDX-FileCopyrightText: 2008 Boudewijn Rempt <boud@valdyas.org>
+ * SPDX-FileCopyrightText: 2009 Sven Langkamp <sven.langkamp@gmail.com>
+ * SPDX-FileCopyrightText: 2011 Silvio Heinrich <plassy@web.de>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public License
- * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #include "kis_curve_option_widget.h"
@@ -50,6 +37,9 @@ KisCurveOptionWidget::KisCurveOptionWidget(KisCurveOption* curveOption, const QS
     m_curveOptionWidget->setupUi(m_widget);
     setConfigurationPage(m_widget);
 
+    strengthToCurveOptionValueScale = 100.0;
+    hideRangeLabelsAndBoxes(true);
+
     m_curveOptionWidget->sensorSelector->setCurveOption(curveOption);
 
     updateSensorCurveLabels(m_curveOptionWidget->sensorSelector->currentHighlighted());
@@ -113,20 +103,26 @@ void KisCurveOptionWidget::readOptionSetting(const KisPropertiesConfigurationSP
 
     m_curveOption->readOptionSetting(setting);
 
-    // Signals needs to be blocked, otherwise checking the checkbox will trigger
-    //   setting the common curve to the widget curve, which is incorrect in this case.
-    bool blockedBefore = m_curveOptionWidget->checkBoxUseSameCurve->blockSignals(true);
-    m_curveOptionWidget->checkBoxUseSameCurve->setChecked(m_curveOption->isSameCurveUsed());
-    m_curveOptionWidget->checkBoxUseSameCurve->blockSignals(blockedBefore);
+    {
+        // Signals needs to be blocked, otherwise checking the checkbox will trigger
+        //   setting the common curve to the widget curve, which is incorrect in this case.
+        KisSignalsBlocker b(m_curveOptionWidget->checkBoxUseSameCurve,
+                            m_curveOptionWidget->checkBoxUseCurve,
+                            m_curveOptionWidget->strengthSlider,
+                            m_curveOptionWidget->curveMode);
 
-    m_curveOptionWidget->checkBoxUseCurve->setChecked(m_curveOption->isCurveUsed());
-    m_curveOptionWidget->strengthSlider->setValue(m_curveOption->value()*100);
-    m_curveOptionWidget->curveMode->setCurrentIndex(m_curveOption->getCurveMode());
+
+        m_curveOptionWidget->checkBoxUseSameCurve->setChecked(m_curveOption->isSameCurveUsed());
+        m_curveOptionWidget->checkBoxUseCurve->setChecked(m_curveOption->isCurveUsed());
+        m_curveOptionWidget->strengthSlider->setValue(m_curveOption->value()*100);
+        m_curveOptionWidget->curveMode->setCurrentIndex(m_curveOption->getCurveMode());
+    }
 
     disableWidgets(!m_curveOption->isCurveUsed());
 
     m_curveOptionWidget->sensorSelector->reload();
-    m_curveOptionWidget->sensorSelector->setCurrent(m_curveOption->activeSensors().first());
+    if(m_curveOption->activeSensors().size() > 0)
+        m_curveOptionWidget->sensorSelector->setCurrent(m_curveOption->activeSensors().first());
     updateSensorCurveLabels(m_curveOptionWidget->sensorSelector->currentHighlighted());
     updateCurve(m_curveOptionWidget->sensorSelector->currentHighlighted());
 }
@@ -151,6 +147,20 @@ void KisCurveOptionWidget::setChecked(bool checked)
     m_curveOption->setChecked(checked);
 }
 
+void KisCurveOptionWidget::setEnabled(bool enabled)
+{
+    m_widget->setEnabled(enabled);
+}
+
+void KisCurveOptionWidget::updateRange(qreal minValue, qreal maxValue)
+{
+    m_curveOption->updateRange(minValue, maxValue);
+
+    // strength settings is shown as 0-100%
+    m_curveOptionWidget->strengthSlider->setRange(m_curveOption->minValue()*100, m_curveOption->maxValue()*100, 0);
+    m_curveOptionWidget->strengthSlider->setValue(m_curveOption->value()*100);
+}
+
 KisCurveOption* KisCurveOptionWidget::curveOption()
 {
     return m_curveOption;
@@ -163,10 +173,11 @@ QWidget* KisCurveOptionWidget::curveWidget()
 
 void KisCurveOptionWidget::slotModified()
 {
-    if (!m_curveOption->isSameCurveUsed()) {
-        m_curveOptionWidget->sensorSelector->currentHighlighted()->setCurve(getWidgetCurve());
+    KisCubicCurve currentCurve = getWidgetCurve();
+    if (m_curveOption->isSameCurveUsed()) {
+        m_curveOption->setCommonCurve(currentCurve);
     } else {
-        m_curveOption->setCommonCurve(getWidgetCurve());
+        m_curveOptionWidget->sensorSelector->currentHighlighted()->setCurve(currentCurve);
     }
     emitSettingChanged();
 }
@@ -175,25 +186,28 @@ void KisCurveOptionWidget::slotUseSameCurveChanged()
 {
     // this is a slot that answers on "Share Curve across all settings" checkbox
     m_curveOption->setUseSameCurve(m_curveOptionWidget->checkBoxUseSameCurve->isChecked());
+    KisCubicCurve currentCurve = getWidgetCurve();
     if (m_curveOption->isSameCurveUsed()) {
-        // !(UseSameCurve) => UseSameCurve
-        // set the current curve to the common curve
-        m_curveOption->setCommonCurve(getWidgetCurve());
+        m_curveOption->setCommonCurve(currentCurve);
     } else {
-        updateCurve(m_curveOptionWidget->sensorSelector->currentHighlighted());
+        for (auto sensor : m_curveOption->activeSensors()) {
+            sensor->setCurve(currentCurve);
+        }
+        // set the current curve since it might be disabled
+        m_curveOptionWidget->sensorSelector->currentHighlighted()->setCurve(currentCurve);
     }
     emitSettingChanged();
 }
 
-void KisCurveOptionWidget::updateSensorCurveLabels(KisDynamicSensorSP sensor)
+void KisCurveOptionWidget::updateSensorCurveLabels(KisDynamicSensorSP sensor) const
 {
     if (sensor) {
-        m_curveOptionWidget->label_xmin->setText(KisDynamicSensor::minimumLabel(sensor->sensorType()));
-        m_curveOptionWidget->label_xmax->setText(KisDynamicSensor::maximumLabel(sensor->sensorType(), sensor->length()));
+        m_curveOptionWidget->label_xmin->setText(sensor->minimumLabel(sensor->sensorType()));
+        m_curveOptionWidget->label_xmax->setText(sensor->maximumLabel(sensor->sensorType(), sensor->length()));
 
-        int inMinValue = KisDynamicSensor::minimumValue(sensor->sensorType());
-        int inMaxValue = KisDynamicSensor::maximumValue(sensor->sensorType(), sensor->length());
-        QString inSuffix = KisDynamicSensor::valueSuffix(sensor->sensorType());
+        int inMinValue = sensor->minimumValue(sensor->sensorType());
+        int inMaxValue = sensor->maximumValue(sensor->sensorType(), sensor->length());
+        QString inSuffix = sensor->valueSuffix(sensor->sensorType());
 
         int outMinValue = m_curveOption->intMinValue();
         int outMaxValue = m_curveOption->intMaxValue();
@@ -225,7 +239,7 @@ void KisCurveOptionWidget::updateLabelsOfCurrentSensor()
 
 void KisCurveOptionWidget::updateValues()
 {
-    m_curveOption->setValue(m_curveOptionWidget->strengthSlider->value()/100.0); // convert back to 0-1 for data
+    m_curveOption->setValue(m_curveOptionWidget->strengthSlider->value()/strengthToCurveOptionValueScale); // convert back to 0-1 for data
     m_curveOption->setCurveUsed(m_curveOptionWidget->checkBoxUseCurve->isChecked());
     disableWidgets(!m_curveOptionWidget->checkBoxUseCurve->isChecked());
     emitSettingChanged();
@@ -346,8 +360,19 @@ void KisCurveOptionWidget::updateThemedIcons()
 
 }
 
+void KisCurveOptionWidget::hideRangeLabelsAndBoxes(bool isHidden) {
 
+    m_curveOptionWidget->xMaxBox->setHidden(isHidden);
+    m_curveOptionWidget->xMinBox->setHidden(isHidden);
+    m_curveOptionWidget->yMaxBox->setHidden(isHidden);
+    m_curveOptionWidget->yMinBox->setHidden(isHidden);
 
+    m_curveOptionWidget->xRangeLabel->setHidden(isHidden);
+    m_curveOptionWidget->yRangeLabel->setHidden(isHidden);
+    m_curveOptionWidget->toLabel1->setHidden(isHidden);
+    m_curveOptionWidget->toLabel2->setHidden(isHidden);
+
+}
 
 KisCubicCurve KisCurveOptionWidget::getWidgetCurve()
 {
