@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2012 Dmitry Kazakov <dimula73@gmail.com>
+ *  SPDX-FileCopyrightText: 2012 Dmitry Kazakov <dimula73@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "move_selection_stroke_strategy.h"
@@ -26,6 +14,7 @@
 #include "kis_painter.h"
 #include "kis_transaction.h"
 #include <commands_new/kis_selection_move_command2.h>
+#include "kis_lod_transform.h"
 
 
 MoveSelectionStrokeStrategy::MoveSelectionStrokeStrategy(KisPaintLayerSP paintLayer,
@@ -47,7 +36,8 @@ MoveSelectionStrokeStrategy::MoveSelectionStrokeStrategy(KisPaintLayerSP paintLa
 }
 
 MoveSelectionStrokeStrategy::MoveSelectionStrokeStrategy(const MoveSelectionStrokeStrategy &rhs)
-    : KisStrokeStrategyUndoCommandBased(rhs),
+    : QObject(),
+      KisStrokeStrategyUndoCommandBased(rhs),
       m_paintLayer(rhs.m_paintLayer),
       m_selection(rhs.m_selection),
       m_updatesFacade(rhs.m_updatesFacade)
@@ -80,10 +70,24 @@ void MoveSelectionStrokeStrategy::initStrokeCallback()
     indirect->setTemporaryTarget(movedDevice);
     indirect->setTemporaryCompositeOp(COMPOSITE_OVER);
     indirect->setTemporaryOpacity(OPACITY_OPAQUE_U8);
+    indirect->setTemporarySelection(0);
+    indirect->setTemporaryChannelFlags(QBitArray());
 
     m_initialDeviceOffset = QPoint(movedDevice->x(), movedDevice->y());
+    m_initialSelectionOffset = QPoint(m_selection->x(), m_selection->y());
 
-    m_selection->setVisible(false);
+    {
+        QRect handlesRect = movedDevice->exactBounds();
+        KisLodTransform t(paintDevice);
+        handlesRect = t.mapInverted(handlesRect);
+
+        if (!handlesRect.isEmpty()) {
+            emit this->sigHandlesRectCalculated(handlesRect);
+        } else {
+            emit this->sigStrokeStartedEmpty();
+        }
+
+    }
 }
 
 void MoveSelectionStrokeStrategy::finishStrokeCallback()
@@ -100,12 +104,12 @@ void MoveSelectionStrokeStrategy::finishStrokeCallback()
 
     indirect->setTemporaryTarget(0);
 
-    QPoint selectionOffset(m_selection->x(), m_selection->y());
-
     m_updatesFacade->blockUpdates();
 
     KUndo2CommandSP moveSelectionCommand(
-        new KisSelectionMoveCommand2(m_selection, selectionOffset, selectionOffset + m_finalOffset));
+        new KisSelectionMoveCommand2(m_selection,
+                                     m_initialSelectionOffset,
+                                     m_initialSelectionOffset + m_finalOffset));
 
     runAndSaveCommand(
         moveSelectionCommand,
@@ -127,13 +131,16 @@ void MoveSelectionStrokeStrategy::cancelStrokeCallback()
     if (indirect) {
         KisPaintDeviceSP t = indirect->temporaryTarget();
         if (t) {
-            QRegion dirtyRegion = t->region();
+            KisRegion dirtyRegion = t->region();
 
             indirect->setTemporaryTarget(0);
 
-            m_selection->setVisible(true);
-
             m_paintLayer->setDirty(dirtyRegion);
+
+            m_selection->setX(m_initialSelectionOffset.x());
+            m_selection->setY(m_initialSelectionOffset.y());
+            m_selection->setVisible(true);
+            m_selection->notifySelectionChanged();
         }
     }
     KisStrokeStrategyUndoCommandBased::cancelStrokeCallback();
@@ -144,6 +151,7 @@ void MoveSelectionStrokeStrategy::cancelStrokeCallback()
 void MoveSelectionStrokeStrategy::doStrokeCallback(KisStrokeJobData *data)
 {
     MoveStrokeStrategy::Data *d = dynamic_cast<MoveStrokeStrategy::Data*>(data);
+    ShowSelectionData *ssd = dynamic_cast<ShowSelectionData*>(data);
 
     if (d) {
         KisIndirectPaintingSupport *indirect =
@@ -151,7 +159,7 @@ void MoveSelectionStrokeStrategy::doStrokeCallback(KisStrokeJobData *data)
 
         KisPaintDeviceSP movedDevice = indirect->temporaryTarget();
 
-        QRegion dirtyRegion = movedDevice->region();
+        QRegion dirtyRegion = movedDevice->region().toQRegion();
 
         QPoint currentDeviceOffset(movedDevice->x(), movedDevice->y());
         QPoint newDeviceOffset(m_initialDeviceOffset + d->offset);
@@ -162,7 +170,17 @@ void MoveSelectionStrokeStrategy::doStrokeCallback(KisStrokeJobData *data)
         movedDevice->setY(newDeviceOffset.y());
         m_finalOffset = d->offset;
 
-        m_paintLayer->setDirty(dirtyRegion);
+        m_paintLayer->setDirty(KisRegion::fromQRegion(dirtyRegion));
+
+        m_selection->setX((m_initialSelectionOffset + d->offset).x());
+        m_selection->setY((m_initialSelectionOffset + d->offset).y());
+
+        if (m_selection->isVisible()) {
+            m_selection->notifySelectionChanged();
+        }
+
+    } else if (ssd) {
+        m_selection->setVisible(ssd->showSelection);
     } else {
         KisStrokeStrategyUndoCommandBased::doStrokeCallback(data);
     }
@@ -172,6 +190,21 @@ KisStrokeStrategy* MoveSelectionStrokeStrategy::createLodClone(int levelOfDetail
 {
     Q_UNUSED(levelOfDetail);
 
+    // Vector selections don't support lod-moves
+    if (m_selection->hasShapeSelection()) return 0;
+
     MoveSelectionStrokeStrategy *clone = new MoveSelectionStrokeStrategy(*this);
+    connect(clone, SIGNAL(sigHandlesRectCalculated(QRect)), this, SIGNAL(sigHandlesRectCalculated(QRect)));
     return clone;
 }
+
+KisStrokeJobData *MoveSelectionStrokeStrategy::ShowSelectionData::createLodClone(int levelOfDetail)
+{
+    return new ShowSelectionData(*this, levelOfDetail);
+}
+
+MoveSelectionStrokeStrategy::ShowSelectionData::ShowSelectionData(const MoveSelectionStrokeStrategy::ShowSelectionData &rhs, int /*levelOfDetail*/)
+    : KisStrokeJobData(rhs),
+      showSelection(rhs.showSelection)
+{
+}
