@@ -1,19 +1,7 @@
 /*
- * Copyright (c) 2017 Boudewijn Rempt <boud@valdyas.org>
+ * SPDX-FileCopyrightText: 2017 Boudewijn Rempt <boud@valdyas.org>
  *
- *  This library is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License as published by
- *  the Free Software Foundation; version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #include "ToolReferenceImages.h"
@@ -25,12 +13,14 @@
 #include <QMessageBox>
 #include <QVector>
 #include <QAction>
+#include <QApplication>
 
 #include <KoSelection.h>
 #include <KoShapeRegistry.h>
 #include <KoShapeManager.h>
 #include <KoShapeController.h>
 #include <KoFileDialog.h>
+#include "KisMimeDatabase.h"
 
 #include <kis_action_registry.h>
 #include <kis_canvas2.h>
@@ -39,6 +29,8 @@
 #include <KisDocument.h>
 #include <KisReferenceImagesLayer.h>
 #include <kis_image.h>
+#include "QClipboard"
+#include "kis_action.h"
 
 #include "ToolReferenceImagesWidget.h"
 #include "KisReferenceImageCollection.h"
@@ -53,9 +45,9 @@ ToolReferenceImages::~ToolReferenceImages()
 {
 }
 
-void ToolReferenceImages::activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes)
+void ToolReferenceImages::activate(const QSet<KoShape*> &shapes)
 {
-    DefaultTool::activate(toolActivation, shapes);
+    DefaultTool::activate(shapes);
 
     auto kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
     connect(kisCanvas->image(), SIGNAL(sigNodeAddedAsync(KisNodeSP)), this, SLOT(slotNodeAdded(KisNodeSP)));
@@ -165,7 +157,7 @@ void ToolReferenceImages::loadReferenceImages()
 
     QFile file(filename);
     if (!file.open(QIODevice::ReadOnly)) {
-        QMessageBox::critical(nullptr, i18nc("@title:window", "Krita"), i18n("Could not open '%1'.", filename));
+        QMessageBox::critical(qApp->activeWindow(), i18nc("@title:window", "Krita"), i18n("Could not open '%1'.", filename));
         return;
     }
 
@@ -187,7 +179,7 @@ void ToolReferenceImages::loadReferenceImages()
         KisDocument *doc = document();
         doc->addCommand(KisReferenceImagesLayer::addReferenceImages(doc, shapes));
     } else {
-        QMessageBox::critical(nullptr, i18nc("@title:window", "Krita"), i18n("Could not load reference images from '%1'.", filename));
+        QMessageBox::critical(qApp->activeWindow(), i18nc("@title:window", "Krita"), i18n("Could not load reference images from '%1'.", filename));
     }
     file.close();
 }
@@ -212,9 +204,14 @@ void ToolReferenceImages::saveReferenceImages()
     QString filename = dialog.filename();
     if (filename.isEmpty()) return;
 
+    QString fileMime = KisMimeDatabase::mimeTypeForFile(filename, false);
+    if (fileMime != "application/x-krita-reference-images") {
+        filename.append(filename.endsWith(".") ? "krf" : ".krf");
+    }
+
     QFile file(filename);
     if (!file.open(QIODevice::WriteOnly)) {
-        QMessageBox::critical(nullptr, i18nc("@title:window", "Krita"), i18n("Could not open '%1' for saving.", filename));
+        QMessageBox::critical(qApp->activeWindow(), i18nc("@title:window", "Krita"), i18n("Could not open '%1' for saving.", filename));
         return;
     }
 
@@ -223,7 +220,7 @@ void ToolReferenceImages::saveReferenceImages()
     file.close();
 
     if (!ok) {
-        QMessageBox::critical(nullptr, i18nc("@title:window", "Krita"), i18n("Failed to save reference images."));
+        QMessageBox::critical(qApp->activeWindow(), i18nc("@title:window", "Krita"), i18n("Failed to save reference images."));
     }
 }
 
@@ -294,6 +291,72 @@ void ToolReferenceImages::deleteSelection()
     }
 }
 
+QMenu* ToolReferenceImages::popupActionsMenu()
+{
+    if (m_contextMenu) {
+        m_contextMenu->clear();
+        m_contextMenu->addSection(i18n("Reference Image Actions"));
+        m_contextMenu->addSeparator();
+
+        QMenu *transform = m_contextMenu->addMenu(i18n("Transform"));
+
+        transform->addAction(action("object_transform_rotate_90_cw"));
+        transform->addAction(action("object_transform_rotate_90_ccw"));
+        transform->addAction(action("object_transform_rotate_180"));
+        transform->addSeparator();
+        transform->addAction(action("object_transform_mirror_horizontally"));
+        transform->addAction(action("object_transform_mirror_vertically"));
+        transform->addSeparator();
+        transform->addAction(action("object_transform_reset"));
+
+        m_contextMenu->addSeparator();
+
+        KisAction* cut = new KisAction(i18n("Cut"));
+        cut->setIcon(KisIconUtils::loadIcon("edit-cut"));
+        KisAction* copy = new KisAction(i18n("Copy"));
+        copy->setIcon(KisIconUtils::loadIcon("edit-copy"));
+        KisAction* paste = new KisAction(i18n("Paste"));
+        paste->setIcon(KisIconUtils::loadIcon("edit-paste"));
+
+        connect(cut,SIGNAL(triggered()),this,SLOT(cut()));
+        connect(copy,SIGNAL(triggered()),this,SLOT(copy()));
+        connect(paste,SIGNAL(triggered()),this,SLOT(paste()));
+
+        m_contextMenu->addAction(cut);
+        m_contextMenu->addAction(copy);
+        m_contextMenu->addAction(paste);
+
+        m_contextMenu->addSeparator();
+
+        m_contextMenu->addAction(action("object_order_front"));
+        m_contextMenu->addAction(action("object_order_raise"));
+        m_contextMenu->addAction(action("object_order_lower"));
+        m_contextMenu->addAction(action("object_order_back"));
+    }
+
+    return m_contextMenu.data();
+}
+
+void ToolReferenceImages::cut()
+{
+    copy();
+    deleteSelection();
+}
+
+void ToolReferenceImages::copy() const
+{
+    QList<KoShape *> shapes = koSelection()->selectedShapes();
+    KoShape* shape = shapes.at(0);
+    KisReferenceImage *reference = dynamic_cast<KisReferenceImage*>(shape);
+    QClipboard *cb = QApplication::clipboard();
+    cb->setImage(reference->getImage());
+}
+
+bool ToolReferenceImages::paste()
+{
+    pasteReferenceImage();
+    return true;
+}
 KisDocument *ToolReferenceImages::document() const
 {
     auto kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
